@@ -16,7 +16,25 @@ import xml.dom.minidom
 import sys
 import re
 import textwrap
-from xml.dom import minidom, Node
+from xml.dom import Node
+import io
+
+messages = {}
+
+# wrapper class to write to file/output in UTF-8 format
+class OutputWriter:
+    def __init__(self,writer) :
+        self.writer = io.open(writer.fileno(), 'w', encoding='utf8')
+
+    def write(self, text) :
+        if sys.version_info.major == 2:
+            self.writer.write(unicode(text))
+        else:
+            self.writer.write(text)
+
+    def flush(self):
+        self.writer.flush()
+
 
 def transformDocs(doc):
     # join lines, unless it is an empty line
@@ -34,12 +52,13 @@ def transformDocs(doc):
     doc = doc.replace("\\$", "$")
     doc = doc.replace("\\#include ", "#include ")
     doc = doc.replace("\\#undef ", "#undef ")
+    doc = doc.replace("\\# ", "# ")
     doc = doc.replace("-# ", "\n - ")
     doc = doc.replace(" - ", "\n - ")
-    doc = doc.replace("\\sa", "\nSee also: ")
-    doc = doc.replace("\\par", "\n")
-    doc = doc.replace("@note", "\nNote:")
-    doc = doc.replace("\\note", "\nNote:")
+    doc = doc.replace("\\sa ", "\nSee also: ")
+    doc = doc.replace("\\par ", "\n")
+    doc = doc.replace("@note ", "\nNote: ")
+    doc = doc.replace("\\note ", "\nNote: ")
     doc = doc.replace("\\verbatim", "\n")
     doc = doc.replace("\\endverbatim", "\n")
     doc = doc.replace("<code>", "")
@@ -50,6 +69,7 @@ def transformDocs(doc):
     doc = doc.replace("\\@", "@")
     doc = doc.replace("\\\\", "\\")
     # \ref name "description" -> description
+    doc = re.sub('\\\\ref +[^ ]* +"\\\\ref"', '\\\\REF', doc)
     doc = re.sub('\\\\ref +[^ ]* +"([^"]*)"', '\\1', doc)
     # \ref specials
     # \ref <key> -> description
@@ -63,7 +83,8 @@ def transformDocs(doc):
                  doc)
     doc = re.sub('\\\\ref +formulas', '"Including formulas"', doc)
     # fallback for not handled
-    doc = re.sub('\\\\ref', '', doc)
+    doc = re.sub('\\\\ref ', ' ', doc)
+    doc = re.sub('\\\\REF', '\\\\ref', doc)
     #<a href="address">description</a> -> description (see: address)
     doc = re.sub('<a +href="([^"]*)" *>([^<]*)</a>', '\\2 (see: \n\\1)', doc)
     # LaTeX name as formula -> LaTeX
@@ -143,8 +164,8 @@ def prepCDocs(node):
     type = node.getAttribute('type')
     format = node.getAttribute('format')
     defval = node.getAttribute('defval')
-    adefval = node.getAttribute('altdefval')
-    doc = "";
+    #adefval = node.getAttribute('altdefval')
+    doc = ""
     if (type != 'obsolete'):
         for n in node.childNodes:
             if (n.nodeName == "docs"):
@@ -153,29 +174,27 @@ def prepCDocs(node):
                         doc += parseDocs(n)
         if (type == 'enum'):
             values = collectValues(node)
-            doc += "<br/>Possible values are: "
+            doc += "<br/>" + messages['possible']
             rng = len(values)
             for i in range(rng):
                 val = values[i]
                 if i == rng - 2:
-                    doc += "%s and " % (val)
+                    doc += "%s%s"%(val,messages['andtxt'])
                 elif i == rng - 1:
                     doc += "%s." % (val)
                 else:
                     doc += "%s, " % (val)
             if (defval != ""):
-                doc += "<br/>The default value is: <code>%s</code>." % (defval)
+                doc += "<br/>" + messages['defvalcode'].format(defval)
         elif (type == 'int'):
             minval = node.getAttribute('minval')
             maxval = node.getAttribute('maxval')
-            doc += "<br/>%s: %s, %s: %s, %s: %s." % (" Minimum value", minval, 
-                     "maximum value", maxval,
-                     "default value", defval)
+            doc += messages['minmaxdef'].format(minval, maxval, defval)
         elif (type == 'bool'):
             if (node.hasAttribute('altdefval')):
-                doc += "<br/>%s: %s." % ("The default value is", "system dependent")
+                doc += "<br/>" + messages['defvaltxt'].format(messages['sysdep'])
             else:
-                doc += "<br/>%s: %s." % ("The default value is", "YES" if (defval == "1") else "NO")
+                doc += "<br/>" + messages['defvaltxt'].format("YES" if (defval == "1") else "NO")
         elif (type == 'list'):
             if format == 'string':
                 values = collectValues(node)
@@ -183,7 +202,7 @@ def prepCDocs(node):
                 for i in range(rng):
                     val = values[i]
                     if i == rng - 2:
-                        doc += "%s and " % (val)
+                        doc += "%s%s"%(val,messages['andtxt'])
                     elif i == rng - 1:
                         doc += "%s." % (val)
                     else:
@@ -191,46 +210,37 @@ def prepCDocs(node):
         elif (type == 'string'):
             if format == 'dir':
                 if defval != '':
-                    doc += "<br/>The default directory is: <code>%s</code>." % (
-                        defval)
+                    doc += "<br/>" + messages['defdir'].format(defval)
             elif format == 'file':
                 abspath = node.getAttribute('abspath')
                 if defval != '':
                     if abspath != '1':
-                        doc += "<br/>The default file is: <code>%s</code>." % (
-                            defval)
+                        doc += "<br/>" + messages['deffile'].format(defval)
                     else:
-                        doc += "<br/>%s: %s%s%s." % (
-                            "The default file (with absolute path) is",
-                            "<code>",defval,"</code>")
+                        doc += "<br/>" + messages['deffileabs'].format(defval)
                 else:
                     if abspath == '1':
-                        doc += "<br/>The file has to be specified with full path."
+                        doc += "<br/>" + messages['deffilefull']
             elif format =='image':
                 abspath = node.getAttribute('abspath')
                 if defval != '':
                     if abspath != '1':
-                        doc += "<br/>The default image is: <code>%s</code>." % (
-                            defval)
+                        doc += "<br/>" + messages['defimg'].format(defval)
                     else:
-                        doc += "<br/>%s: %s%s%s." % (
-                            "The default image (with absolute path) is",
-                            "<code>",defval,"</code>")
+                        doc += "<br/>" + messages['defimgabs'].format(defval)
                 else:
                     if abspath == '1':
-                        doc += "<br/>The image has to be specified with full path."
+                        doc += "<br/>" + messages['defimgfull']
             else: # format == 'string':
                 if defval != '':
-                    doc += "<br/>The default value is: <code>%s</code>." % (
-                        defval)
+                    doc += "<br/>" + messages['defvalcode'].format(defval)
         # depends handling
         if (node.hasAttribute('depends')):
             depends = node.getAttribute('depends')
-            doc += "<br/>%s \\ref cfg_%s \"%s\" is set to \\c YES." % (
-                "This tag requires that the tag", depends.lower(), depends.upper())
+            doc += "<br/>" + messages['depstxt'].format(depends.lower(), depends.upper())
 
     docC = transformDocs(doc)
-    return docC;
+    return docC
 
 
 def parseOption(node):
@@ -245,7 +255,7 @@ def parseOption(node):
     depends = node.getAttribute('depends')
     setting = node.getAttribute('setting')
     orgtype = node.getAttribute('orgtype')
-    docC = prepCDocs(node);
+    docC = prepCDocs(node)
     if len(setting) > 0:
         print("#if %s" % (setting))
     print("  //----")
@@ -329,10 +339,16 @@ def parseOption(node):
         rng = len(docC)
         for i in range(rng):
             line = docC[i]
-            if i != rng - 1:  # since we go from 0 to rng-1
-                print("              \"%s\\n\"" % (line))
-            else:
-                print("              \"%s\"" % (line))
+            try:
+                if i != rng - 1:  # since we go from 0 to rng-1
+                    print("              \"%s\\n\"" % (line))
+                else:
+                    print("              \"%s\"" % (line))
+            except Exception as inst:
+                sys.stdout = sys.stderr
+                print("")
+                print(inst)
+                print("")
         print("             );")
         addValues("cl", node)
         if depends != '':
@@ -455,7 +471,7 @@ def parseGroupMapAvailable(node):
             if type=='enum':
                 if len(setting) > 0:
                     print("#if %s" % (setting))
-                print("    %-22s isAvailable_%-41s { return v.lower() == %s_enum2str(%s_str2enum(v)).lower(); }" % ('bool',name+'(QCString v)',name,name));
+                print("    %-22s isAvailable_%-41s { return v.lower() == %s_enum2str(%s_str2enum(v)).lower(); }" % ('bool',name+'(QCString v)',name,name))
                 if len(setting) > 0:
                     print("#endif")
 
@@ -497,7 +513,7 @@ def getEnum2BoolMapping(node):
             bool_rep = nv.getAttribute("bool_representation")
             if name and bool_rep:
                 bool_value = "true" if bool_rep and bool_rep.upper() == 'YES' else "false"
-                mapping.append( "{{ \"{0}\", \"{1}\" }}".format(escape(name),bool_value))
+                mapping.append( "{{ \"{0}\", {1} }}".format(escape(name),bool_value))
     return mapping
 
 def parseGroupMapInit(node):
@@ -523,7 +539,7 @@ def parseGroupCDocs(node):
         if n.nodeType == Node.ELEMENT_NODE:
             type = n.getAttribute('type')
             name = n.getAttribute('id')
-            docC = prepCDocs(n);
+            docC = prepCDocs(n)
             if type != 'obsolete':
                 print("  doc->add(")
                 print("              \"%s\"," % (name))
@@ -542,9 +558,9 @@ def parseOptionDoc(node, first):
     type = node.getAttribute('type')
     format = node.getAttribute('format')
     defval = node.getAttribute('defval')
-    adefval = node.getAttribute('altdefval')
+    #adefval = node.getAttribute('altdefval')
     depends = node.getAttribute('depends')
-    setting = node.getAttribute('setting')
+    #setting = node.getAttribute('setting')
     doc = ""
     if (type != 'obsolete'):
         for n in node.childNodes:
@@ -565,12 +581,12 @@ def parseOptionDoc(node, first):
         if (type == 'enum'):
             values = collectValues(node)
             print("")
-            print("Possible values are: ")
+            print(messages['possible'])
             rng = len(values)
             for i in range(rng):
                 val = values[i]
                 if i == rng - 2:
-                    print("%s and " % (val))
+                    print("%s%s" % (val,messages['andtxt']))
                 elif i == rng - 1:
                     print("%s." % (val))
                 else:
@@ -578,26 +594,22 @@ def parseOptionDoc(node, first):
             if (defval != ""):
                 print("")
                 print("")
-                print("The default value is: <code>%s</code>." % (defval))
+                print(messages['defvalcode'].format(defval))
             print("")
         elif (type == 'int'):
             minval = node.getAttribute('minval')
             maxval = node.getAttribute('maxval')
             print("")
             print("")
-            print("%s: %s%s%s, %s: %s%s%s, %s: %s%s%s." % (
-                     " Minimum value", "<code>", minval, "</code>", 
-                     "maximum value", "<code>", maxval, "</code>",
-                     "default value", "<code>", defval, "</code>"))
+            print(messages['minmaxdefcode'].format(minval, maxval,defval))
             print("")
         elif (type == 'bool'):
             print("")
             print("")
             if (node.hasAttribute('altdefval')):
-                print("The default value is: system dependent.")
+                print(messages['defvaltxt'].format(messages['sysdep']))
             else:
-                print("The default value is: <code>%s</code>." % (
-                    "YES" if (defval == "1") else "NO"))
+                print(messages['defvalcode'].format("YES" if (defval == "1") else "NO"))
             print("")
         elif (type == 'list'):
             if format == 'string':
@@ -606,7 +618,7 @@ def parseOptionDoc(node, first):
                 for i in range(rng):
                     val = values[i]
                     if i == rng - 2:
-                        print("%s and " % (val))
+                        print("%s%s" % (val,messages['andtxt']))
                     elif i == rng - 1:
                         print("%s." % (val))
                     else:
@@ -616,57 +628,48 @@ def parseOptionDoc(node, first):
             if format == 'dir':
                 if defval != '':
                     print("")
-                    print("The default directory is: <code>%s</code>." % (
-                        defval))
+                    print(messages['defdir'].format(defval))
             elif format == 'file':
                 abspath = node.getAttribute('abspath')
                 if defval != '':
                     print("")
                     if abspath != '1':
-                        print("The default file is: <code>%s</code>." % (
-                            defval))
+                        print(messages['deffile'].format(defval))
                     else:
-                        print("%s: %s%s%s." % (
-                            "The default file (with absolute path) is",
-                            "<code>",defval,"</code>"))
+                        print(messages['deffileabs'].format(defval))
                 else:
                     if abspath == '1':
                         print("")
-                        print("The file has to be specified with full path.")
+                        print(messages['deffilefull'])
             elif format =='image':
                 abspath = node.getAttribute('abspath')
                 if defval != '':
                     print("")
                     if abspath != '1':
-                        print("The default image is: <code>%s</code>." % (
-                            defval))
+                        print(messages['defimg'].format(defval))
                     else:
-                        print("%s: %s%s%s." % (
-                            "The default image (with absolute path) is",
-                            "<code>",defval,"</code>"))
+                        print(messages['defimgabs'].format(defval))
                 else:
                     if abspath == '1':
                         print("")
-                        print("The image has to be specified with full path.")
+                        print(messages['defimgfull'])
             else: # format == 'string':
                 if defval != '':
                     print("")
-                    print("The default value is: <code>%s</code>." % (
-                        defval.replace('\\','\\\\')))
+                    print(messages['defvalcode'].format(defval.replace('\\','\\\\')))
             print("")
         # depends handling
         if (node.hasAttribute('depends')):
             depends = node.getAttribute('depends')
             print("")
-            print("%s \\ref cfg_%s \"%s\" is set to \\c YES." % (
-                "This tag requires that the tag", depends.lower(), depends.upper()))
+            print(messages['depstxt'].format(depends.lower(), depends.upper()))
         return False
 
 
 def parseGroupsDoc(node):
     name = node.getAttribute('name')
     doc = node.getAttribute('docs')
-    print("\section config_%s %s" % (name.lower(), doc))
+    print("\\section config_%s %s" % (name.lower(), doc))
     # Start of list has been moved to the first option for better
     # anchor placement
     #  print "<dl>"
@@ -680,7 +683,6 @@ def parseGroupsDoc(node):
 
 
 def parseGroupsList(node, commandsList):
-    list = ()
     for n in node.childNodes:
         if n.nodeType == Node.ELEMENT_NODE:
             type = n.getAttribute('type')
@@ -694,7 +696,7 @@ def parseDocs(node):
     for n in node.childNodes:
         if n.nodeType == Node.TEXT_NODE:
             doc += n.nodeValue.strip()
-        if n.nodeType == Node.CDATA_SECTION_NODE:
+        elif n.nodeType == Node.CDATA_SECTION_NODE:
             doc += n.nodeValue.rstrip("\r\n ").lstrip("\r\n")
     #doc += "<br>"
     return doc
@@ -720,11 +722,30 @@ def parseFooterDoc(node):
     print(doc)
 
 
+def parseGenerator(node):
+    for n in node.childNodes:
+        if n.nodeType == Node.ELEMENT_NODE:
+            if (n.nodeName == "message"):
+                name = n.getAttribute('name')
+                doc = ""
+                for n1 in n.childNodes:
+                    if n1.nodeType == Node.TEXT_NODE:
+                        doc += n1.nodeValue.strip()
+                    elif n1.nodeType == Node.CDATA_SECTION_NODE:
+                        doc += n1.nodeValue.rstrip("\r\n").lstrip("\r\n")
+                messages[name] = doc
+
 def main():
-    if len(sys.argv)<3 or (not sys.argv[1] in ['-doc','-cpp','-wiz','-maph','-maps']):
+    if len(sys.argv)<3 or (sys.argv[1] not in ['-doc','-cpp','-wiz','-maph','-maps']):
         sys.exit('Usage: %s -doc|-cpp|-wiz|-maph|-maps config.xml' % sys.argv[0])
     try:
-        doc = xml.dom.minidom.parse(sys.argv[2])
+        configFile = sys.argv[2]
+        if sys.version_info.major == 2:
+            fh = open(configFile,'r')
+        else:
+            fh = open(configFile,'r',encoding='utf8')
+        sys.stdout = OutputWriter(sys.stdout)
+        doc = xml.dom.minidom.parse(fh)
     except Exception as inst:
         sys.stdout = sys.stderr
         print("")
@@ -732,6 +753,14 @@ def main():
         print("")
         sys.exit(1)
     elem = doc.documentElement
+
+    for n in elem.childNodes:
+        if n.nodeType == Node.ELEMENT_NODE:
+            if (n.nodeName == "generator"):
+                parseGenerator(n)
+    if len(messages)==0:
+        sys.exit('<generator> section missing in %s' % configFile)
+
     if (sys.argv[1] == "-doc"):
         print("/* WARNING: This file is generated!")
         print(" * Do not edit this file, but edit config.xml instead and run")
@@ -801,10 +830,10 @@ def main():
         print("    struct Info")
         print("    {")
         print("      enum Type { Bool, Int, String, List, Unknown };")
-        print("      using Enum2BoolMap = std::unordered_map<std::string,bool>;");
+        print("      using Enum2BoolMap = std::unordered_map<std::string,bool>;")
         print("      Info(Type t,bool         ConfigValues::*b) : type(t), value(b) {}")
         print("      Info(Type t,int          ConfigValues::*i) : type(t), value(i) {}")
-        print("      Info(Type t,QCString     ConfigValues::*s, Enum2BoolMap boolMap = {}) : type(t), value(s), m_boolMap(boolMap) {}")
+        print("      Info(Type t,QCString     ConfigValues::*s, const Enum2BoolMap &boolMap = {}) : type(t), value(s), m_boolMap(boolMap) {}")
         print("      Info(Type t,StringVector ConfigValues::*l) : type(t), value(l) {}")
         print("      Type type;")
         print("      union Item")
@@ -841,18 +870,18 @@ def main():
         print("#include \"configimpl.h\"")
         print("#include <unordered_map>")
         print("")
-        print("const ConfigValues::Info *ConfigValues::get(const QCString &tag) const");
-        print("{");
-        print("  static const std::unordered_map< std::string, Info > configMap =");
-        print("  {");
+        print("const ConfigValues::Info *ConfigValues::get(const QCString &tag) const")
+        print("{")
+        print("  static const std::unordered_map< std::string, Info > configMap =")
+        print("  {")
         for n in elem.childNodes:
             if n.nodeType == Node.ELEMENT_NODE:
                 if (n.nodeName == "group"):
                     parseGroupMapInit(n)
-        print("  };");
-        print("  auto it = configMap.find(tag.str());");
-        print("  return it!=configMap.end() ? &it->second : nullptr;");
-        print("}");
+        print("  };")
+        print("  auto it = configMap.find(tag.str());")
+        print("  return it!=configMap.end() ? &it->second : nullptr;")
+        print("}")
         print("")
         print("void ConfigValues::init()")
         print("{")
@@ -868,7 +897,7 @@ def main():
         print("")
         print("StringVector ConfigValues::fields() const")
         print("{")
-        print("  return {");
+        print("  return {")
         first=True
         for n in elem.childNodes:
             if n.nodeType == Node.ELEMENT_NODE:
@@ -893,7 +922,7 @@ def main():
         print("    auto it = m_boolMap.find((ConfigValues::instance().*(value.s)).str());")
         print("    if (it!=m_boolMap.end())")
         print("    {")
-        print("      return it->second;");
+        print("      return it->second;")
         print("    }")
         print("  }")
         print("  return false;")
@@ -912,11 +941,11 @@ def main():
         print("")
         print("void addConfigOptions(ConfigImpl *cfg)")
         print("{")
-        print("  ConfigString *cs;")
-        print("  ConfigEnum   *ce;")
-        print("  ConfigList   *cl;")
-        print("  ConfigInt    *ci;")
-        print("  ConfigBool   *cb;")
+        print("  ConfigString *cs = nullptr;")
+        print("  ConfigEnum   *ce = nullptr;")
+        print("  ConfigList   *cl = nullptr;")
+        print("  ConfigInt    *ci = nullptr;")
+        print("  ConfigBool   *cb = nullptr;")
         print("")
         # process header
         for n in elem.childNodes:
