@@ -29,6 +29,7 @@
 #include "qcstring.h"
 #include "config.h"
 #include "construct.h"
+#include "message.h"
 
 class TextStream;
 
@@ -136,7 +137,7 @@ class ConfigList : public ConfigOption
     WidgetType widgetType() const { return m_widgetType; }
     StringVector *valueRef() { return &m_value; }
     StringVector getDefault() { return m_defaultValue; }
-    void emptyValueToDefault() override { if (m_value.empty() && !m_defaultValue.empty()) m_value=m_defaultValue; };
+    void emptyValueToDefault() override { if (m_value.empty() && !m_defaultValue.empty()) m_value=m_defaultValue; }
     void writeTemplate(TextStream &t,bool sl,bool) override;
     void compareDoxyfile(TextStream &t,Config::CompareMode compareMode) override;
     void writeXMLDoxyfile(TextStream &t) override;
@@ -204,7 +205,7 @@ class ConfigString : public ConfigOption
     void writeXSDDoxyfile(TextStream &t) override;
     void substEnvVars() override;
     void init() override { m_value = m_defValue; }
-    void emptyValueToDefault() override { if (m_value.isEmpty()) m_value=m_defValue; };
+    void emptyValueToDefault() override { if (m_value.isEmpty()) m_value=m_defValue; }
     bool isDefault() override { return m_value.stripWhiteSpace() == m_defValue.stripWhiteSpace(); }
 
   private:
@@ -363,32 +364,32 @@ class ConfigImpl
      *  @{
      */
 
-    /*! Returns the value of the string option with name \a fileName.
-     *  The arguments \a num and \a name are for debugging purposes only.
+    /*! Returns the value of the string option with name \a name.
+     *  The arguments \a num and \a fileName are for debugging purposes only.
      *  There is a convenience function Config_getString() for this.
      */
     QCString &getString(const char *fileName,int num,const char *name) const;
 
-    /*! Returns the value of the list option with name \a fileName.
-     *  The arguments \a num and \a name are for debugging purposes only.
+    /*! Returns the value of the list option with name \a name.
+     *  The arguments \a num and \a fileName are for debugging purposes only.
      *  There is a convenience function Config_getList() for this.
      */
     StringVector &getList(const char *fileName,int num,const char *name) const;
 
-    /*! Returns the value of the enum option with name \a fileName.
-     *  The arguments \a num and \a name are for debugging purposes only.
+    /*! Returns the value of the enum option with name \a name.
+     *  The arguments \a num and \a fileName are for debugging purposes only.
      *  There is a convenience function Config_getEnum() for this.
      */
     QCString &getEnum(const char *fileName,int num,const char *name) const;
 
-    /*! Returns the value of the integer option with name \a fileName.
-     *  The arguments \a num and \a name are for debugging purposes only.
+    /*! Returns the value of the integer option with name \a name.
+     *  The arguments \a num and \a fileName are for debugging purposes only.
      *  There is a convenience function Config_getInt() for this.
      */
     int      &getInt(const char *fileName,int num,const char *name) const;
 
-    /*! Returns the value of the boolean option with name \a fileName.
-     *  The arguments \a num and \a name are for debugging purposes only.
+    /*! Returns the value of the boolean option with name \a name.
+     *  The arguments \a num and \a fileName are for debugging purposes only.
      *  There is a convenience function Config_getBool() for this.
      */
     bool     &getBool(const char *fileName,int num,const char *name) const;
@@ -426,7 +427,7 @@ class ConfigImpl
     {
       ConfigString *result = new ConfigString(name,doc);
       m_options.push_back(std::unique_ptr<ConfigOption>(result));
-      m_dict.insert(std::make_pair(name,result));
+      m_dict.emplace(name,result);
       return result;
     }
 
@@ -440,7 +441,7 @@ class ConfigImpl
     {
       ConfigEnum *result = new ConfigEnum(name,doc,defVal);
       m_options.push_back(std::unique_ptr<ConfigOption>(result));
-      m_dict.insert(std::make_pair(name,result));
+      m_dict.emplace(name,result);
       return result;
     }
 
@@ -452,7 +453,7 @@ class ConfigImpl
     {
       ConfigList *result = new ConfigList(name,doc);
       m_options.push_back(std::unique_ptr<ConfigOption>(result));
-      m_dict.insert(std::make_pair(name,result));
+      m_dict.emplace(name,result);
       return result;
     }
 
@@ -467,7 +468,7 @@ class ConfigImpl
     {
       ConfigInt *result = new ConfigInt(name,doc,minVal,maxVal,defVal);
       m_options.push_back(std::unique_ptr<ConfigOption>(result));
-      m_dict.insert(std::make_pair(name,result));
+      m_dict.emplace(name,result);
       return result;
     }
 
@@ -481,23 +482,25 @@ class ConfigImpl
     {
       ConfigBool *result = new ConfigBool(name,doc,defVal);
       m_options.push_back(std::unique_ptr<ConfigOption>(result));
-      m_dict.insert(std::make_pair(name,result));
+      m_dict.emplace(name,result);
       return result;
     }
+
     /*! Adds an option that has become obsolete. */
     ConfigOption *addObsolete(const char *name,ConfigOption::OptionType orgType)
     {
       ConfigObsolete *result = new ConfigObsolete(name,orgType);
       m_obsolete.push_back(std::unique_ptr<ConfigOption>(result));
-      m_dict.insert(std::make_pair(name,result));
+      m_dict.emplace(name,result);
       return result;
     }
+
     /*! Adds an option that has been disabled at compile time. */
     ConfigOption *addDisabled(const char *name)
     {
       ConfigDisabled *result = new ConfigDisabled(name);
       m_disabled.push_back(std::unique_ptr<ConfigOption>(result));
-      m_dict.insert(std::make_pair(name,result));
+      m_dict.emplace(name,result);
       return result;
     }
     /*! @} */
@@ -604,9 +607,27 @@ class ConfigImpl
       return substitute(result,"\r","");
     }
 
-    static void config_err(const char *fmt, ...);
-    static void config_term(const char *fmt, ...);
-    static void config_warn(const char *fmt, ...);
+    static void config_err_ (fmt::string_view fmt, fmt::format_args args);
+    static void config_term_(fmt::string_view fmt, fmt::format_args args);
+    static void config_warn_(fmt::string_view fmt, fmt::format_args args);
+
+    template<typename ...Args>
+    static void config_err(fmt::format_string<Args...> fmt, Args&&... args)
+    {
+      config_err_(fmt,fmt::make_format_args(args...));
+    }
+
+    template<typename ...Args>
+    static void config_term(fmt::format_string<Args...> fmt, Args&&... args)
+    {
+      config_term_(fmt,fmt::make_format_args(args...));
+    }
+
+    template<typename ...Args>
+    static void config_warn(fmt::format_string<Args...> fmt, Args&&... args)
+    {
+      config_warn_(fmt,fmt::make_format_args(args...));
+    }
 
   private:
     ConfigOptionList m_options;
