@@ -50,7 +50,6 @@
 #include "dirdef.h"
 #include "section.h"
 #include "dir.h"
-#include "growbuf.h"
 #include "outputlist.h"
 #include "moduledef.h"
 
@@ -308,8 +307,11 @@ DB_GEN_C1(*m_t)
 void DocbookCodeGenerator::endCodeFragment(const QCString &)
 {
 DB_GEN_C1(*m_t)
+  bool wasHidden = m_hide;
+  m_hide = false;
   //endCodeLine checks is there is still an open code line, if so closes it.
   endCodeLine();
+  m_hide = wasHidden;
 
   *m_t << "</programlisting>";
 }
@@ -389,7 +391,7 @@ void DocbookGenerator::cleanup()
 }
 
 
-void DocbookGenerator::startFile(const QCString &name,const QCString &,const QCString &,int,int)
+void DocbookGenerator::startFile(const QCString &name,bool,const QCString &,const QCString &,int,int)
 {
 DB_GEN_C
   QCString fileName=name;
@@ -614,7 +616,7 @@ DB_GEN_C2("IndexSection " << is)
         {
           if (dd->isLinkableInProject())
           {
-            m_t << "<    xi:include href=\"" << dd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
+            m_t << "    <xi:include href=\"" << dd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
           }
         }
       }
@@ -627,7 +629,7 @@ DB_GEN_C2("IndexSection " << is)
         {
           if (nd->isLinkableInProject() && !nd->isAlias())
           {
-            m_t << "<xi:include href=\"" << nd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
+            m_t << "    <xi:include href=\"" << nd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
           }
         }
       }
@@ -640,7 +642,7 @@ DB_GEN_C2("IndexSection " << is)
         {
           if (cd->isLinkableInProject() && !cd->isAlias())
           {
-            m_t << "<xi:include href=\"" << cd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
+            m_t << "    <xi:include href=\"" << cd->getOutputFileBase() << ".xml\" xmlns:xi=\"http://www.w3.org/2001/XInclude\"/>\n";
           }
         }
       }
@@ -723,11 +725,11 @@ DB_GEN_C
     m_pageLinks += link;
 }
 
-void DocbookGenerator::writeDoc(const IDocNodeAST *ast,const Definition *ctx,const MemberDef *,int)
+void DocbookGenerator::writeDoc(const IDocNodeAST *ast,const Definition *ctx,const MemberDef *,int,int sectionLevel)
 {
 DB_GEN_C
   auto astImpl = dynamic_cast<const DocNodeAST*>(ast);
-  if (astImpl)
+  if (astImpl && sectionLevel<=m_tocState.maxLevel)
   {
     DocbookDocVisitor visitor(m_t,*m_codeList,ctx?ctx->getDefFileExtension():QCString());
     std::visit(visitor,astImpl->root);
@@ -1482,7 +1484,8 @@ static constexpr auto hex="0123456789ABCDEF";
 QCString convertToDocBook(const QCString &s, const bool retainNewline)
 {
   if (s.isEmpty()) return s;
-  GrowBuf growBuf;
+  QCString result;
+  result.reserve(s.length()+32);
   const char *p = s.data();
   const char *q = nullptr;
   int cnt = 0;
@@ -1491,9 +1494,15 @@ QCString convertToDocBook(const QCString &s, const bool retainNewline)
   {
     switch (c)
     {
-      case '\n': if (retainNewline) growBuf.addStr("<literallayout>&#160;&#xa;</literallayout>"); growBuf.addChar(c);   break;
-      case '<':  growBuf.addStr("&lt;");   break;
-      case '>':  growBuf.addStr("&gt;");   break;
+      case '\n':
+        if (retainNewline) 
+        {
+          result+="<literallayout>&#160;&#xa;</literallayout>";
+          result+=c;
+        }
+        break;
+      case '<':  result+="&lt;";   break;
+      case '>':  result+="&gt;";   break;
       case '&':  // possibility to have a special symbol
         q = p;
         cnt = 2; // we have to count & and ; as well
@@ -1509,37 +1518,36 @@ QCString convertToDocBook(const QCString &s, const bool retainNewline)
            if (res == HtmlEntityMapper::Sym_Unknown)
            {
              p++;
-             growBuf.addStr("&amp;");
+             result+="&amp;";
            }
            else
            {
-             growBuf.addStr(HtmlEntityMapper::instance().docbook(res));
+             result+=HtmlEntityMapper::instance().docbook(res);
              q++;
              p = q;
            }
         }
         else
         {
-          growBuf.addStr("&amp;");
+          result+="&amp;";
         }
         break;
-      case '\'': growBuf.addStr("&apos;"); break;
-      case '"':  growBuf.addStr("&quot;"); break;
+      case '\'': result+="&apos;"; break;
+      case '"':  result+="&quot;"; break;
       case  1: case  2: case  3: case  4: case  5: case  6: case 7:  case  8:
       case 11: case 12: case 14: case 15: case 16: case 17: case 18:
       case 19: case 20: case 21: case 22: case 23: case 24: case 25: case 26:
       case 27: case 28: case 29: case 30: case 31:
-        growBuf.addStr("&#x24");
-        growBuf.addChar(hex[static_cast<uint8_t>(c)>>4]);
-        growBuf.addChar(hex[static_cast<uint8_t>(c)&0xF]);
-        growBuf.addChar(';');
+        result+="&#x24";
+        result+=hex[static_cast<uint8_t>(c)>>4];
+        result+=hex[static_cast<uint8_t>(c)&0xF];
+        result+=';';
         break;
       default:
-        growBuf.addChar(c);
+        result+=c;
         break;
     }
   }
-  growBuf.addChar(0);
-  return growBuf.get();
+  return result;
 }
 
