@@ -63,6 +63,7 @@ class GroupDefImpl : public DefinitionMixin<GroupDef>
     QCString anchor() const override { return QCString(); }
     QCString displayName(bool=TRUE) const override { return hasGroupTitle() ? m_title : DefinitionMixin::name(); }
     QCString groupTitle() const override { return m_title; }
+    QCString groupTitleAsText() const override { return m_titleAsText; }
     void setGroupTitle( const QCString &newtitle ) override;
     bool hasGroupTitle( ) const override { return m_titleSet; }
     void addFile(FileDef *def) override;
@@ -95,6 +96,7 @@ class GroupDefImpl : public DefinitionMixin<GroupDef>
     void findSectionsInDocumentation() override;
 
     void addListReferences() override;
+    void addRequirementReferences() override;
     void sortMemberLists() override;
     bool subGrouping() const override { return m_subGrouping; }
 
@@ -151,6 +153,7 @@ class GroupDefImpl : public DefinitionMixin<GroupDef>
     void setGroupTitleLocal( const QCString &title);
 
     QCString             m_title;               // title of the group
+    QCString             m_titleAsText;         // title of the group in plain text
     bool                 m_titleSet;            // true if title is not the same as the name
     QCString             m_fileName;            // base name of the generated file
     FileList             m_fileList;            // list of files in the group
@@ -164,7 +167,7 @@ class GroupDefImpl : public DefinitionMixin<GroupDef>
     DirList              m_dirList;             // list of directories in the group
     MemberList           m_allMemberList;
     MemberNameInfoLinkedMap m_allMemberNameInfoLinkedMap;
-    Definition *         m_groupScope;
+    Definition *         m_groupScope = nullptr;
     MemberLists          m_memberLists;
     MemberGroupList      m_memberGroups;
     bool                 m_subGrouping;
@@ -210,12 +213,14 @@ void GroupDefImpl::setGroupTitleLocal( const QCString &t )
   if ( !t.isEmpty())
   {
     m_title = t;
+    m_titleAsText = parseCommentAsText(this,nullptr,t,docFile(),docLine());
     m_titleSet = TRUE;
   }
   else
   {
     m_title = name();
     m_title[0]=static_cast<char>(toupper(m_title[0]));
+    m_titleAsText = m_title;
     m_titleSet = FALSE;
   }
 }
@@ -379,9 +384,9 @@ bool GroupDefImpl::insertMember(MemberDef *md,bool docOnly)
 
     if (srcMd->isFunction() && md->isFunction() && // both are a function
         (tSrcMdAl.size()==tMdAl.size()) &&       // same number of template arguments
-        matchArguments2(srcMd->getOuterScope(),srcMd->getFileDef(),&srcMdAl,
-                        md->getOuterScope(),md->getFileDef(),&mdAl,
-                        TRUE,srcMd->getLanguage()
+        matchArguments2(srcMd->getOuterScope(),srcMd->getFileDef(),srcMd->typeString(),&srcMdAl,
+                           md->getOuterScope(),   md->getFileDef(),   md->typeString(),&mdAl,
+                           TRUE,srcMd->getLanguage()
                        ) && // matching parameters
         sameScope // both are found in the same scope
        )
@@ -651,7 +656,7 @@ void GroupDefImpl::writeTagFile(TextStream &tagFile)
   addHtmlExtensionIfMissing(fn);
   tagFile << "  <compound kind=\"group\">\n";
   tagFile << "    <name>" << convertToXML(name()) << "</name>\n";
-  tagFile << "    <title>" << convertToXML(m_title) << "</title>\n";
+  tagFile << "    <title>" << convertToXML(m_titleAsText) << "</title>\n";
   tagFile << "    <filename>" << fn << "</filename>\n";
   for (const auto &lde : LayoutDocManager::instance().docEntries(LayoutDocManager::Group))
   {
@@ -781,9 +786,7 @@ void GroupDefImpl::writeTagFile(TextStream &tagFile)
 
 void GroupDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title)
 {
-  if ((!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF))
-      || !documentation().isEmpty() || !inbodyDocumentation().isEmpty()
-     )
+  if (hasDetailedDescription())
   {
     ol.pushGeneratorState();
     if (m_pages.size()!=numDocMembers()) // not only pages -> classical layout
@@ -807,10 +810,15 @@ void GroupDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title
     ol.popGeneratorState();
 
     // repeat brief description
+    ol.startTextBlock();
     if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF))
     {
-      ol.generateDoc(briefFile(),briefLine(),this,nullptr,briefDescription(),FALSE,FALSE,
-                     QCString(),FALSE,FALSE);
+      ol.generateDoc(briefFile(),
+                     briefLine(),
+                     this,
+                     nullptr,
+                     briefDescription(),
+                     DocOptions());
     }
     // write separator between brief and details
     if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF) &&
@@ -829,16 +837,28 @@ void GroupDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title
     // write detailed documentation
     if (!documentation().isEmpty())
     {
-      ol.generateDoc(docFile(),docLine(),this,nullptr,documentation()+"\n",TRUE,FALSE,
-                     QCString(),FALSE,FALSE);
+      ol.generateDoc(docFile(),
+                     docLine(),
+                     this,
+                     nullptr,
+                     documentation()+"\n",
+                     DocOptions()
+                     .setIndexWords(true));
     }
 
     // write inbody documentation
     if (!inbodyDocumentation().isEmpty())
     {
-      ol.generateDoc(inbodyFile(),inbodyLine(),this,nullptr,inbodyDocumentation()+"\n",TRUE,FALSE,
-                     QCString(),FALSE,FALSE);
+      ol.generateDoc(inbodyFile(),
+                     inbodyLine(),
+                     this,
+                     nullptr,
+                     inbodyDocumentation()+"\n",
+                     DocOptions()
+                     .setIndexWords(true));
     }
+    if (hasRequirementRefs()) writeRequirementRefs(ol);
+    ol.endTextBlock();
   }
 }
 
@@ -848,9 +868,15 @@ void GroupDefImpl::writeBriefDescription(OutputList &ol)
   {
     auto parser { createDocParser() };
     auto ast    { validatingParseDoc(*parser.get(),
-                                     briefFile(),briefLine(),this,nullptr,
-                                     briefDescription(),TRUE,FALSE,
-                                     QCString(),TRUE,FALSE) };
+                                     briefFile(),
+                                     briefLine(),
+                                     this,
+                                     nullptr,
+                                     briefDescription(),
+                                     DocOptions()
+                                     .setIndexWords(true)
+                                     .setSingleLine(true))
+                };
     if (!ast->isEmpty())
     {
       ol.startParagraph();
@@ -864,7 +890,7 @@ void GroupDefImpl::writeBriefDescription(OutputList &ol)
       ol.writeString(" \n");
       ol.enable(OutputType::RTF);
 
-      if (hasDetailedDescription())
+      if (hasDetailedDescription() && m_pages.size()!=numDocMembers()) // group with non-page members
       {
         ol.disableAllBut(OutputType::Html);
         ol.startTextLink(QCString(),"details");
@@ -926,8 +952,13 @@ void GroupDefImpl::writeFiles(OutputList &ol,const QCString &title)
       if (!fd->briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
       {
         ol.startMemberDescription(fd->getOutputFileBase());
-        ol.generateDoc(briefFile(),briefLine(),fd,nullptr,fd->briefDescription(),FALSE,FALSE,
-                       QCString(),TRUE,FALSE);
+        ol.generateDoc(briefFile(),
+                       briefLine(),
+                       fd,
+                       nullptr,
+                       fd->briefDescription(),
+                       DocOptions()
+                       .setSingleLine(true));
         ol.endMemberDescription();
       }
       ol.endMemberDeclaration(QCString(),QCString());
@@ -966,13 +997,18 @@ void GroupDefImpl::writeNestedGroups(OutputList &ol,const QCString &title)
         if (anc.isEmpty()) anc=gd->name(); else anc.prepend(gd->name()+"_");
         ol.startMemberItem(anc,OutputGenerator::MemberItemType::Normal);
         ol.insertMemberAlign();
-        ol.writeObjectLink(gd->getReference(),gd->getOutputFileBase(),QCString(),gd->groupTitle());
+        ol.writeObjectLink(gd->getReference(),gd->getOutputFileBase(),QCString(),gd->groupTitleAsText());
         ol.endMemberItem(OutputGenerator::MemberItemType::Normal);
         if (!gd->briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
         {
           ol.startMemberDescription(gd->getOutputFileBase());
-          ol.generateDoc(briefFile(),briefLine(),gd,nullptr,gd->briefDescription(),FALSE,FALSE,
-                         QCString(),TRUE,FALSE);
+          ol.generateDoc(briefFile(),
+                         briefLine(),
+                         gd,
+                         nullptr,
+                         gd->briefDescription(),
+                         DocOptions()
+                         .setSingleLine(true));
           ol.endMemberDescription();
         }
         ol.endMemberDeclaration(QCString(),QCString());
@@ -1005,8 +1041,13 @@ void GroupDefImpl::writeDirs(OutputList &ol,const QCString &title)
       if (!dd->briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
       {
         ol.startMemberDescription(dd->getOutputFileBase());
-        ol.generateDoc(briefFile(),briefLine(),dd,nullptr,dd->briefDescription(),FALSE,FALSE,
-                       QCString(),TRUE,FALSE);
+        ol.generateDoc(briefFile(),
+                       briefLine(),
+                       dd,
+                       nullptr,
+                       dd->briefDescription(),
+                       DocOptions()
+                       .setSingleLine(true));
         ol.endMemberDescription();
       }
       ol.endMemberDeclaration(QCString(),QCString());
@@ -1055,8 +1096,13 @@ void GroupDefImpl::writePageDocumentation(OutputList &ol)
         ol.endSection(si->label(),SectionType::Subsection);
       }
       ol.startTextBlock();
-      ol.generateDoc(pd->docFile(),pd->docLine(),pd,nullptr,(pd->documentation()+pd->inbodyDocumentation()),TRUE,FALSE,
-                     QCString(),TRUE,FALSE);
+      ol.generateDoc(pd->docFile(),
+                     pd->docLine(),
+                     pd,
+                     nullptr,
+                     (pd->documentation()+pd->inbodyDocumentation()),
+                     DocOptions()
+                     .setIndexWords(true));
       ol.endTextBlock();
     }
   }
@@ -1185,7 +1231,7 @@ void GroupDefImpl::writeDocumentation(OutputList &ol)
     ++hierarchyLevel;
   }
 
-  startFile(ol,getOutputFileBase(),name(),m_title,HighlightedItem::Topics,
+  startFile(ol,getOutputFileBase(),false,name(),m_titleAsText,HighlightedItem::Topics,
             FALSE /* additionalIndices*/, QCString() /*altSidebarName*/, hierarchyLevel);
 
   ol.startHeaderSection();
@@ -1195,25 +1241,32 @@ void GroupDefImpl::writeDocumentation(OutputList &ol)
   //1.{
   ol.pushGeneratorState();
   ol.disable(OutputType::Man);
-  ol.generateDoc(docFile(),getStartBodyLine(),this,nullptr,m_title,TRUE,FALSE,
-                 QCString(),TRUE,FALSE,Config_getBool(MARKDOWN_SUPPORT),false);
+  ol.generateDoc(docFile(),
+                 getStartBodyLine(),
+                 this,
+                 nullptr,
+                 m_title,
+                 DocOptions()
+                 .setIndexWords(true)
+                 .setSingleLine(true)
+                 .setAutolinkSupport(false));
   ol.popGeneratorState();
   //1.}
   addGroupListToTitle(ol,this);
   //2.{
   ol.pushGeneratorState();
   ol.disable(OutputType::Man);
-  ol.endTitleHead(getOutputFileBase(),m_title);
+  ol.endTitleHead(getOutputFileBase(),m_titleAsText);
   ol.popGeneratorState();
   //2.}
   //3.{
   ol.pushGeneratorState();
   ol.disableAllBut(OutputType::Man);
   ol.endTitleHead(getOutputFileBase(),name());
-  if (!m_title.isEmpty())
+  if (!m_titleAsText.isEmpty())
   {
     ol.writeString(" - ");
-    ol.parseText(m_title);
+    ol.parseText(m_titleAsText);
   }
   ol.popGeneratorState();
   //3.}
@@ -1749,16 +1802,13 @@ QCString GroupDefImpl::getOutputFileBase() const
 
 void GroupDefImpl::addListReferences()
 {
-  {
-    const RefItemVector &xrefItems = xrefListItems();
-    addRefItem(xrefItems,
+  addRefItem(xrefListItems(),
              getOutputFileBase(),
              theTranslator->trGroup(TRUE,TRUE),
              getOutputFileBase(),name(),
              QCString(),
              nullptr
             );
-  }
   for (const auto &mg : m_memberGroups)
   {
     mg->addListReferences(this);
@@ -1772,6 +1822,22 @@ void GroupDefImpl::addListReferences()
   }
 }
 
+void GroupDefImpl::addRequirementReferences()
+{
+  RequirementManager::instance().addRequirementRefsForSymbol(this);
+  for (const auto &mg : m_memberGroups)
+  {
+    mg->addRequirementReferences(this);
+  }
+  for (auto &ml : m_memberLists)
+  {
+    if (ml->listType().isDocumentation())
+    {
+      ml->addRequirementReferences(this);
+    }
+  }
+}
+
 void GroupDefImpl::addMemberToList(MemberListType lt,MemberDef *md)
 {
   bool sortBriefDocs = Config_getBool(SORT_BRIEF_DOCS);
@@ -1867,6 +1933,24 @@ void GroupDefImpl::sortMemberLists()
     };
 
     std::stable_sort(m_namespaces.begin(),m_namespaces.end(),namespaceComp);
+
+    auto moduleComp = [](const ModuleLinkedRefMap::Ptr &m1,const ModuleLinkedRefMap::Ptr &m2)
+    {
+      return qstricmp_sort(m1->name(),m2->name())<0;
+    };
+
+    std::stable_sort(m_modules.begin(), m_modules.end(), moduleComp);
+
+    auto conceptComp = [](const ConceptLinkedRefMap::Ptr &c1,const ConceptLinkedRefMap::Ptr &c2)
+    {
+      return qstricmp_sort(c1->name(),c2->name())<0;
+    };
+
+    std::stable_sort(m_concepts.begin(), m_concepts.end(), conceptComp);
+
+    std::stable_sort(m_dirList.begin(), m_dirList.end(), compareDirDefs);
+    std::stable_sort(m_fileList.begin(), m_fileList.end(), compareFileDefs);
+
   }
   else
   {
@@ -1977,8 +2061,8 @@ bool GroupDefImpl::hasDetailedDescription() const
   bool repeatBrief = Config_getBool(REPEAT_BRIEF);
   return ((!briefDescription().isEmpty() && repeatBrief) ||
          !documentation().isEmpty() ||
-         !inbodyDocumentation().isEmpty()) &&
-         (m_pages.size()!=numDocMembers());
+         !inbodyDocumentation().isEmpty() ||
+         hasRequirementRefs());
 }
 
 void GroupDefImpl::overrideGroupGraph(bool e)
