@@ -41,6 +41,7 @@
 #include "portable.h"
 #include "codefragment.h"
 #include "cite.h"
+#include "md5.h"
 
 static const int g_maxLevels = 7;
 static const std::array<const char *,g_maxLevels> g_secLabels =
@@ -111,18 +112,18 @@ static void visitPreStart(TextStream &t, bool hasCaption, QCString name,  QCStri
 {
     if (inlineImage)
     {
-      t << "\n\\begin{DoxyInlineImage}\n";
+      t << "\n\\begin{DoxyInlineImage}%\n";
     }
     else
     {
       if (hasCaption)
       {
-        t << "\n\\begin{DoxyImage}\n";
+        t << "\n\\begin{DoxyImage}%\n";
       }
       else
       {
-        t << "\n\\begin{DoxyImageNoCaption}\n"
-               "  \\mbox{";
+        t << "\n\\begin{DoxyImageNoCaption}%\n"
+               "  \\doxymbox{";
       }
     }
 
@@ -170,11 +171,11 @@ static void visitPreStart(TextStream &t, bool hasCaption, QCString name,  QCStri
       {
         if (Config_getBool(PDF_HYPERLINKS))
         {
-          t << "\n\\doxyfigcaption{";
+          t << "%\n\\doxyfigcaption{";
         }
         else
         {
-          t << "\n\\doxyfigcaptionnolink{";
+          t << "%\n\\doxyfigcaptionnolink{";
         }
       }
       else
@@ -190,11 +191,11 @@ static void visitPostEnd(TextStream &t, bool hasCaption, bool inlineImage = FALS
 {
     if (inlineImage)
     {
-      t << "\n\\end{DoxyInlineImage}\n";
+      t << "%\n\\end{DoxyInlineImage}\n";
     }
     else
     {
-      t << "}\n"; // end mbox or caption
+      t << "}%\n"; // end doxymbox or caption
       if (hasCaption)
       {
         t << "\\end{DoxyImage}\n";
@@ -206,29 +207,6 @@ static void visitPostEnd(TextStream &t, bool hasCaption, bool inlineImage = FALS
     }
 }
 
-static QCString makeShortName(const QCString &name)
-{
-  QCString shortName = name;
-  int i = shortName.findRev('/');
-  if (i!=-1)
-  {
-    shortName=shortName.mid(i+1);
-  }
-  return shortName;
-}
-
-static QCString makeBaseName(const QCString &name)
-{
-  QCString baseName = makeShortName(name);
-  int i=baseName.find('.');
-  if (i!=-1)
-  {
-    baseName=baseName.left(i);
-  }
-  return baseName;
-}
-
-
 void LatexDocVisitor::visitCaption(const DocNodeList &children)
 {
   for (const auto &n : children)
@@ -237,34 +215,6 @@ void LatexDocVisitor::visitCaption(const DocNodeList &children)
   }
 }
 
-QCString LatexDocVisitor::escapeMakeIndexChars(const char *s)
-{
-  QCString result;
-  const char *p=s;
-  char str[2]; str[1]=0;
-  char c = 0;
-  if (p)
-  {
-    while ((c=*p++))
-    {
-      switch (c)
-      {
-        case '!': m_t << "\"!"; break;
-        case '"': m_t << "\"\""; break;
-        case '@': m_t << "\"@"; break;
-        case '|': m_t << "\\texttt{\"|}"; break;
-        case '[': m_t << "["; break;
-        case ']': m_t << "]"; break;
-        case '{': m_t << "\\lcurly{}"; break;
-        case '}': m_t << "\\rcurly{}"; break;
-        default:  str[0]=c; filter(str); break;
-      }
-    }
-  }
-  return result;
-}
-
-
 LatexDocVisitor::LatexDocVisitor(TextStream &t,OutputCodeList &ci,LatexCodeGenerator &lcg,
                                  const QCString &langExt, int hierarchyLevel)
   : m_t(t), m_ci(ci), m_lcg(lcg), m_insidePre(FALSE),
@@ -378,7 +328,14 @@ void LatexDocVisitor::operator()(const DocURL &u)
 void LatexDocVisitor::operator()(const DocLineBreak &)
 {
   if (m_hide) return;
+  if (m_insideItem)
+  {
+  m_t << "\\\\\n";
+  }
+  else
+  {
   m_t << "~\\newline\n";
+  }
 }
 
 void LatexDocVisitor::operator()(const DocHorRuler &)
@@ -463,7 +420,7 @@ void LatexDocVisitor::operator()(const DocVerbatim &s)
         m_ci.startCodeFragment("DoxyCode");
         getCodeParser(lang).parseCode(m_ci,s.context(),s.text(),langExt,
                                       Config_getBool(STRIP_CODE_COMMENTS),
-                                      s.isExample(),s.exampleFile());
+                                      CodeParserOptions().setExample(s.isExample(),s.exampleFile()));
         m_ci.endCodeFragment("DoxyCode");
       }
       break;
@@ -476,9 +433,18 @@ void LatexDocVisitor::operator()(const DocVerbatim &s)
       m_t << "}";
       break;
     case DocVerbatim::Verbatim:
-      m_t << "\\begin{DoxyVerb}";
-      m_t << s.text();
-      m_t << "\\end{DoxyVerb}\n";
+      if (isTableNested(s.parent())) // in table
+      {
+        m_t << "\\begin{DoxyCode}{0}";
+        filter(s.text(), true);
+        m_t << "\\end{DoxyCode}\n";
+      }
+      else
+      {
+        m_t << "\\begin{DoxyVerb}";
+        m_t << s.text();
+        m_t << "\\end{DoxyVerb}\n";
+      }
       break;
     case DocVerbatim::HtmlOnly:
     case DocVerbatim::XmlOnly:
@@ -492,70 +458,44 @@ void LatexDocVisitor::operator()(const DocVerbatim &s)
       break;
     case DocVerbatim::Dot:
       {
-        static int dotindex = 1;
-        QCString fileName(4096, QCString::ExplicitSize);
-
-        fileName.sprintf("%s%d%s",
-            qPrint(Config_getString(LATEX_OUTPUT)+"/inline_dotgraph_"),
-            dotindex++,
-            ".dot"
-           );
-        std::ofstream file = Portable::openOutputStream(fileName);
-        if (!file.is_open())
+        bool exists = false;
+        auto fileName = writeFileContents(Config_getString(LATEX_OUTPUT)+"/inline_dotgraph_", // baseName
+                                          ".dot",                                             // extension
+                                          s.text(),                                           // contents
+                                          exists);
+        if (!fileName.isEmpty())
         {
-          err("Could not open file {} for writing\n",fileName);
-        }
-        else
-        {
-          file.write( s.text().data(), s.text().length() );
-          file.close();
-
-          startDotFile(fileName,s.width(),s.height(),s.hasCaption(),s.srcFile(),s.srcLine());
+          startDotFile(fileName,s.width(),s.height(),s.hasCaption(),s.srcFile(),s.srcLine(),!exists);
           visitChildren(s);
           endDotFile(s.hasCaption());
-
-          if (Config_getBool(DOT_CLEANUP)) Dir().remove(fileName.str());
         }
       }
       break;
     case DocVerbatim::Msc:
       {
-        static int mscindex = 1;
-        QCString baseName(4096, QCString::ExplicitSize);
-
-        baseName.sprintf("%s%d",
-            qPrint(Config_getString(LATEX_OUTPUT)+"/inline_mscgraph_"),
-            mscindex++
-           );
-        QCString fileName = baseName+".msc";
-        std::ofstream file = Portable::openOutputStream(fileName);
-        if (!file.is_open())
+        bool exists = false;
+        auto fileName = writeFileContents(Config_getString(LATEX_OUTPUT)+"/inline_mscgraph_", // baseName
+                                          ".msc",                                             // extension
+                                          "msc {"+s.text()+"}",                               // contents
+                                          exists);
+        if (!fileName.isEmpty())
         {
-          err("Could not open file {} for writing\n",fileName);
-        }
-        else
-        {
-          QCString text = "msc {";
-          text+=s.text();
-          text+="}";
-          file.write( text.data(), text.length() );
-          file.close();
-
-          writeMscFile(baseName, s);
-
-          if (Config_getBool(DOT_CLEANUP)) Dir().remove(fileName.str());
+          writeMscFile(fileName, s, !exists);
         }
       }
       break;
     case DocVerbatim::PlantUML:
       {
         QCString latexOutput = Config_getString(LATEX_OUTPUT);
-        QCString baseName = PlantumlManager::instance().writePlantUMLSource(
+        auto baseNameVector = PlantumlManager::instance().writePlantUMLSource(
                               latexOutput,s.exampleFile(),s.text(),
                               s.useBitmap() ? PlantumlManager::PUML_BITMAP : PlantumlManager::PUML_EPS,
                               s.engine(),s.srcFile(),s.srcLine(),true);
 
-        writePlantUMLFile(baseName, s);
+        for (const auto &baseName: baseNameVector)
+        {
+          writePlantUMLFile(baseName, s);
+        }
       }
       break;
   }
@@ -587,14 +527,10 @@ void LatexDocVisitor::operator()(const DocInclude &inc)
                                                   inc.text(),
                                                   langExt,
                                                   inc.stripCodeComments(),
-                                                  inc.isExample(),
-                                                  inc.exampleFile(),
-                                                  fd.get(),    // fileDef,
-                                                  -1,    // start line
-                                                  -1,    // end line
-                                                  FALSE, // inline fragment
-                                                  nullptr,     // memberDef
-                                                  TRUE   // show line numbers
+                                                  CodeParserOptions()
+                                                  .setExample(inc.isExample(), inc.exampleFile())
+                                                  .setFileDef(fd.get())
+                                                  .setInlineFragment(true)
        				                 );
         m_ci.endCodeFragment("DoxyCodeInclude");
       }
@@ -605,14 +541,10 @@ void LatexDocVisitor::operator()(const DocInclude &inc)
         getCodeParser(inc.extension()).parseCode(m_ci,inc.context(),
                                                   inc.text(),langExt,
                                                   inc.stripCodeComments(),
-                                                  inc.isExample(),
-                                                  inc.exampleFile(),
-                                                  nullptr,     // fileDef
-                                                  -1,    // startLine
-                                                  -1,    // endLine
-                                                  TRUE,  // inlineFragment
-                                                  nullptr,     // memberDef
-                                                  FALSE
+                                                  CodeParserOptions()
+                                                  .setExample(inc.isExample(), inc.exampleFile())
+                                                  .setInlineFragment(true)
+                                                  .setShowLineNumbers(false)
                                                  );
         m_ci.endCodeFragment("DoxyCodeInclude");
       }
@@ -629,9 +561,18 @@ void LatexDocVisitor::operator()(const DocInclude &inc)
       m_t << inc.text();
       break;
     case DocInclude::VerbInclude:
-      m_t << "\n\\begin{DoxyVerbInclude}\n";
-      m_t << inc.text();
-      m_t << "\\end{DoxyVerbInclude}\n";
+      if (isTableNested(inc.parent())) // in table
+      {
+        m_t << "\\begin{DoxyCode}{0}";
+        filter(inc.text(), true);
+        m_t << "\\end{DoxyCode}\n";
+      }
+      else
+      {
+        m_t << "\n\\begin{DoxyVerbInclude}\n";
+        m_t << inc.text();
+        m_t << "\\end{DoxyVerbInclude}\n";
+      }
       break;
     case DocInclude::Snippet:
     case DocInclude::SnippetWithLines:
@@ -678,13 +619,11 @@ void LatexDocVisitor::operator()(const DocIncOperator &op)
 
       getCodeParser(locLangExt).parseCode(m_ci,op.context(),op.text(),langExt,
                                           op.stripCodeComments(),
-                                          op.isExample(),op.exampleFile(),
-                                          fd.get(),     // fileDef
-                                          op.line(),    // startLine
-                                          -1,    // endLine
-                                          FALSE, // inline fragment
-                                          nullptr,     // memberDef
-                                          op.showLineNo()  // show line numbers
+                                          CodeParserOptions()
+                                          .setExample(op.isExample(),op.exampleFile())
+                                          .setFileDef(fd.get())
+                                          .setStartLine(op.line())
+                                          .setShowLineNumbers(op.showLineNo())
                                          );
     }
     pushHidden(m_hide);
@@ -723,11 +662,7 @@ void LatexDocVisitor::operator()(const DocFormula &f)
 void LatexDocVisitor::operator()(const DocIndexEntry &i)
 {
   if (m_hide) return;
-  m_t << "\\index{";
-  m_t << latexEscapeLabelName(i.entry());
-  m_t << "@{";
-  m_t << latexEscapeIndexChars(i.entry());
-  m_t << "}}";
+  latexWriteIndexItem(m_t,i.entry());
 }
 
 void LatexDocVisitor::operator()(const DocSimpleSectSep &)
@@ -1222,11 +1157,11 @@ void LatexDocVisitor::operator()(const DocHtmlDescList &dl)
 void LatexDocVisitor::operator()(const DocHtmlDescTitle &dt)
 {
   if (m_hide) return;
-  m_t << "\n\\item[";
+  m_t << "\n\\item[{\\parbox[t]{\\linewidth}{";
   m_insideItem=TRUE;
   visitChildren(dt);
   m_insideItem=FALSE;
-  m_t << "]";
+  m_t << "}}]";
 }
 
 void LatexDocVisitor::operator()(const DocHtmlDescData &dd)
@@ -1253,26 +1188,24 @@ void LatexDocVisitor::writeStartTableCommand(const DocNodeVariant *n,size_t cols
 {
   if (isTableNested(n))
   {
-    m_t << "{\\begin{tabularx}{\\linewidth}{|*{" << cols << "}{>{\\raggedright\\arraybackslash}X|}}";
+    m_t << "\\begin{DoxyTableNested}{" << cols << "}";
   }
   else
   {
-    m_t << "\\tabulinesep=1mm\n\\begin{longtabu}spread 0pt [c]{*{" << cols << "}{|X[-1]}|}\n";
+    m_t << "\n\\begin{DoxyTable}{" << cols << "}";
   }
-  //return isNested ? "TabularNC" : "TabularC";
 }
 
 void LatexDocVisitor::writeEndTableCommand(const DocNodeVariant *n)
 {
   if (isTableNested(n))
   {
-    m_t << "\\end{tabularx}}\n";
+    m_t << "\\end{DoxyTableNested}\n";
   }
   else
   {
-    m_t << "\\end{longtabu}\n";
+    m_t << "\\end{DoxyTable}\n";
   }
-  //return isNested ? "TabularNC" : "TabularC";
 }
 
 void LatexDocVisitor::operator()(const DocHtmlTable &t)
@@ -1292,32 +1225,29 @@ void LatexDocVisitor::operator()(const DocHtmlTable &t)
   }
 
   writeStartTableCommand(t.parent(),t.numColumns());
-
-  if (c)
+  if (!isTableNested(t.parent()))
   {
-    m_t << "\\caption{";
-    std::visit(*this, *t.caption());
+    // write caption
+    m_t << "{";
+    if (c)
+    {
+      std::visit(*this, *t.caption());
+    }
+    m_t << "}";
+    // write label
+    m_t << "{";
+    if (c && (!stripPath(c->file()).isEmpty() || !c->anchor().isEmpty()))
+    {
+      m_t << stripPath(c->file()) << "_" << c->anchor();
+    }
     m_t << "}";
-    m_t << "\\label{" << stripPath(c->file()) << "_" << c->anchor() << "}";
-    m_t << "\\\\\n";
   }
 
+  // write head row(s)
+  m_t << "{" << t.numberHeaderRows() << "}\n";
+
   setNumCols(t.numColumns());
-  m_t << "\\hline\n";
 
-  // check if first row is a heading and then render the row already here
-  // and end it with \endfirsthead (triggered via m_firstRow==TRUE)
-  // then repeat the row as normal and end it with \endhead (m_firstRow==FALSE)
-  const DocHtmlRow *firstRow = std::get_if<DocHtmlRow>(t.firstRow());
-  if (firstRow && firstRow->isHeading())
-  {
-    setFirstRow(TRUE);
-    if (!isTableNested(t.parent()))
-    {
-      std::visit(*this,*t.firstRow());
-    }
-    setFirstRow(FALSE);
-  }
   visitChildren(t);
   writeEndTableCommand(t.parent());
   popTableState();
@@ -1336,32 +1266,6 @@ void LatexDocVisitor::operator()(const DocHtmlRow &row)
 
   visitChildren(row);
 
-  size_t c=currentColumn();
-  while (c<=numCols()) // end of row while inside a row span?
-  {
-    for (const auto &span : rowSpans())
-    {
-      //printf("  found row span: column=%d rs=%d cs=%d rowIdx=%d cell->rowIdx=%d i=%d c=%d\n",
-      //    span->column, span->rowSpan,span->colSpan,row.rowIndex(),span->cell->rowIndex(),i,c);
-      if (span.rowSpan>0 && span.column==c &&  // we are at a cell in a row span
-          row.rowIndex()>span.cell.rowIndex() // but not the row that started the span
-         )
-      {
-        m_t << "&";
-        if (span.colSpan>1) // row span is also part of a column span
-        {
-          m_t << "\\multicolumn{" << span.colSpan << "}{";
-          m_t <<  "}|}{}";
-        }
-        else // solitary row span
-        {
-          m_t << "\\multicolumn{1}{c|}{}";
-        }
-      }
-    }
-    c++;
-  }
-
   m_t << "\\\\";
 
   size_t col = 1;
@@ -1374,7 +1278,6 @@ void LatexDocVisitor::operator()(const DocHtmlRow &row)
     }
     else if (span.column>col)
     {
-      m_t << "\\cline{" << col << "-" << (span.column-1) << "}";
       col = span.column+span.colSpan;
     }
     else
@@ -1383,142 +1286,115 @@ void LatexDocVisitor::operator()(const DocHtmlRow &row)
     }
   }
 
-  if (col <= numCols())
-  {
-    m_t << "\\cline{" << col << "-" << numCols() << "}";
-  }
-
   m_t << "\n";
-
-  const DocNodeVariant *n = ::parent(row.parent());
-  if (row.isHeading() && row.rowIndex()==1 && !isTableNested(n))
-  {
-    if (firstRow())
-    {
-      m_t << "\\endfirsthead\n";
-      m_t << "\\hline\n";
-      m_t << "\\endfoot\n";
-      m_t << "\\hline\n";
-    }
-    else
-    {
-      m_t << "\\endhead\n";
-    }
-  }
 }
 
 void LatexDocVisitor::operator()(const DocHtmlCell &c)
 {
   if (m_hide) return;
-
-  const DocHtmlRow *row = std::get_if<DocHtmlRow>(c.parent());
+  //printf("Cell(r=%u,c=%u) rowSpan=%d colSpan=%d currentColumn()=%zu\n",c.rowIndex(),c.columnIndex(),c.rowSpan(),c.colSpan(),currentColumn());
 
   setCurrentColumn(currentColumn()+1);
 
-  //Skip columns that span from above.
+  QCString cellOpts;
+  QCString cellSpec;
+  auto appendOpt = [&cellOpts](const QCString &s)
+  {
+    if (!cellOpts.isEmpty()) cellOpts+=",";
+    cellOpts+=s;
+  };
+  auto appendSpec = [&cellSpec](const QCString &s)
+  {
+    if (!cellSpec.isEmpty()) cellSpec+=",";
+    cellSpec+=s;
+  };
+  auto writeCell = [this,&cellOpts,&cellSpec]()
+  {
+    if (!cellOpts.isEmpty() || !cellSpec.isEmpty())
+    {
+      m_t << "\\SetCell";
+      if (!cellOpts.isEmpty())
+      {
+        m_t << "[" << cellOpts << "]";
+      }
+      m_t << "{" << cellSpec << "}";
+    }
+  };
+
+  // skip over columns that have a row span starting at an earlier row
   for (const auto &span : rowSpans())
   {
+    //printf("span(r=%u,c=%u): column=%zu colSpan=%zu,rowSpan=%zu currentColumn()=%zu\n",
+    //    span.cell.rowIndex(),span.cell.columnIndex(),
+    //    span.column,span.colSpan,span.rowSpan,
+    //    currentColumn());
     if (span.rowSpan>0 && span.column==currentColumn())
     {
-      if (row && span.colSpan>1)
+      setCurrentColumn(currentColumn()+span.colSpan);
+      for (size_t i=0;i<span.colSpan;i++)
       {
-        m_t << "\\multicolumn{" << span.colSpan << "}{";
-        if (currentColumn() /*c.columnIndex()*/==1) // add extra | for first column
-        {
-          m_t << "|";
-        }
-        m_t << "l|}{" << (c.isHeading()? "\\columncolor{\\tableheadbgcolor}" : "") << "}"; // alignment not relevant, empty column
-        setCurrentColumn(currentColumn()+span.colSpan);
+        m_t << "&";
       }
-      else
-      {
-        setCurrentColumn(currentColumn()+1);
-      }
-      m_t << "&";
     }
   }
 
   int cs = c.colSpan();
-  int a = c.alignment();
-  if (cs>1 && row)
-  {
-    setInColSpan(TRUE);
-    m_t << "\\multicolumn{" << cs << "}{";
-    if (c.columnIndex()==1) // add extra | for first column
-    {
-      m_t << "|";
-    }
-    switch (a)
-    {
-      case DocHtmlCell::Right:
-        m_t << "r|}{";
-        break;
-      case DocHtmlCell::Center:
-        m_t << "c|}{";
-        break;
-      default:
-        m_t << "l|}{";
-        break;
-    }
-  }
+  int ha = c.alignment();
   int rs = c.rowSpan();
   int va = c.valignment();
-  if (rs>0)
+
+  switch (ha) // horizontal alignment
   {
-    setInRowSpan(TRUE);
-    m_t << "\\multirow";
-    switch(va)
-    {
-      case DocHtmlCell::Top:
-        m_t << "[t]";
-        break;
-      case DocHtmlCell::Bottom:
-        m_t << "[b]";
-        break;
-      case DocHtmlCell::Middle:
-        break; // No alignment option needed
-      default:
-        break;
-    }
-    //printf("adding row span: cell={r=%d c=%d rs=%d cs=%d} curCol=%d\n",
+    case DocHtmlCell::Right:
+      appendSpec("r");
+      break;
+    case DocHtmlCell::Center:
+      appendSpec("c");
+      break;
+    default:
+      // default
+      break;
+  }
+  if (rs>0) // row span
+  {
+    appendOpt("r="+QCString().setNum(rs));
+    //printf("adding row span: cell={r=%d c=%d rs=%d cs=%d} curCol=%zu\n",
     //                       c.rowIndex(),c.columnIndex(),c.rowSpan(),c.colSpan(),
     //                       currentColumn());
     addRowSpan(ActiveRowSpan(c,rs,cs,currentColumn()));
-    m_t << "{" << rs << "}{*}{";
   }
-  if (a==DocHtmlCell::Center)
+  if (cs>1) // column span
   {
-    m_t << "\\PBS\\centering ";
-  }
-  else if (a==DocHtmlCell::Right)
-  {
-    m_t << "\\PBS\\raggedleft ";
+    // update column to the end of the span, needs to be done *after* calling addRowSpan()
+    setCurrentColumn(currentColumn()+cs-1);
+    appendOpt("c="+QCString().setNum(cs));
   }
   if (c.isHeading())
   {
-    m_t << "\\cellcolor{\\tableheadbgcolor}\\textbf{ ";
+    appendSpec("bg=\\tableheadbgcolor");
+    appendSpec("font=\\bfseries");
   }
-  if (cs>1)
+  switch(va) // vertical alignment
   {
-    setCurrentColumn(currentColumn()+cs-1);
+    case DocHtmlCell::Top:
+      appendSpec("h");
+      break;
+    case DocHtmlCell::Bottom:
+      appendSpec("f");
+      break;
+    case DocHtmlCell::Middle:
+      // default
+      break;
   }
+  writeCell();
 
   visitChildren(c);
 
-  if (c.isHeading())
+  for (int i=0;i<cs-1;i++)
   {
-    m_t << "}";
-  }
-  if (inRowSpan())
-  {
-    setInRowSpan(FALSE);
-    m_t << "}";
-  }
-  if (inColSpan())
-  {
-    setInColSpan(FALSE);
-    m_t << "}";
+    m_t << "&"; // placeholder for invisible cell
   }
+
   if (!c.isLast()) m_t << "&";
 }
 
@@ -1602,28 +1478,61 @@ void LatexDocVisitor::operator()(const DocImage &img)
 void LatexDocVisitor::operator()(const DocDotFile &df)
 {
   if (m_hide) return;
-  if (!Config_getBool(DOT_CLEANUP)) copyFile(df.file(),Config_getString(LATEX_OUTPUT)+"/"+stripPath(df.file()));
-  startDotFile(df.file(),df.width(),df.height(),df.hasCaption(),df.srcFile(),df.srcLine());
-  visitChildren(df);
-  endDotFile(df.hasCaption());
+  bool exists = false;
+  std::string inBuf;
+  if (readInputFile(df.file(),inBuf))
+  {
+    auto fileName = writeFileContents(Config_getString(LATEX_OUTPUT)+"/"+stripPath(df.file())+"_", // baseName
+                                      ".dot",                                                      // extension
+                                      inBuf,                                                       // contents
+                                      exists);
+    if (!fileName.isEmpty())
+    {
+      startDotFile(fileName,df.width(),df.height(),df.hasCaption(),df.srcFile(),df.srcLine(),!exists);
+      visitChildren(df);
+      endDotFile(df.hasCaption());
+    }
+  }
 }
 
 void LatexDocVisitor::operator()(const DocMscFile &df)
 {
   if (m_hide) return;
-  if (!Config_getBool(DOT_CLEANUP)) copyFile(df.file(),Config_getString(LATEX_OUTPUT)+"/"+stripPath(df.file()));
-  startMscFile(df.file(),df.width(),df.height(),df.hasCaption(),df.srcFile(),df.srcLine());
-  visitChildren(df);
-  endMscFile(df.hasCaption());
+  bool exists = false;
+  std::string inBuf;
+  if (readInputFile(df.file(),inBuf))
+  {
+    auto fileName = writeFileContents(Config_getString(LATEX_OUTPUT)+"/"+stripPath(df.file())+"_", // baseName
+                                      ".msc",                                                      // extension
+                                      inBuf,                                                      // contents
+                                      exists);
+    if (!fileName.isEmpty())
+    {
+      startMscFile(fileName,df.width(),df.height(),df.hasCaption(),df.srcFile(),df.srcLine(),!exists);
+      visitChildren(df);
+      endMscFile(df.hasCaption());
+    }
+  }
 }
 
 void LatexDocVisitor::operator()(const DocDiaFile &df)
 {
   if (m_hide) return;
-  if (!Config_getBool(DOT_CLEANUP)) copyFile(df.file(),Config_getString(LATEX_OUTPUT)+"/"+stripPath(df.file()));
-  startDiaFile(df.file(),df.width(),df.height(),df.hasCaption(),df.srcFile(),df.srcLine());
-  visitChildren(df);
-  endDiaFile(df.hasCaption());
+  bool exists = false;
+  std::string inBuf;
+  if (readInputFile(df.file(),inBuf))
+  {
+    auto fileName = writeFileContents(Config_getString(LATEX_OUTPUT)+"/"+stripPath(df.file())+"_", // baseName
+                                      ".dia",                                                      // extension
+                                      inBuf,                                                      // contents
+                                      exists);
+    if (!fileName.isEmpty())
+    {
+      startDiaFile(fileName,df.width(),df.height(),df.hasCaption(),df.srcFile(),df.srcLine(),!exists);
+      visitChildren(df);
+      endDiaFile(df.hasCaption());
+    }
+  }
 }
 
 void LatexDocVisitor::operator()(const DocPlantUmlFile &df)
@@ -1796,7 +1705,7 @@ void LatexDocVisitor::operator()(const DocParamList &pl)
   {
     if (pl.direction()!=DocParamSect::Unspecified)
     {
-      m_t << "\\mbox{\\texttt{";
+      m_t << "\\doxymbox{\\texttt{";
       if (pl.direction()==DocParamSect::In)
       {
         m_t << "in";
@@ -1863,7 +1772,7 @@ void LatexDocVisitor::operator()(const DocXRefItem &x)
   m_t << "\\item[";
   if (pdfHyperlinks && !anonymousEnum)
   {
-    m_t << "\\mbox{\\hyperlink{" << stripPath(x.file()) << "_" << x.anchor() << "}{";
+    m_t << "\\doxymbox{\\hyperlink{" << stripPath(x.file()) << "_" << x.anchor() << "}{";
   }
   else
   {
@@ -2007,14 +1916,13 @@ void LatexDocVisitor::startDotFile(const QCString &fileName,
                                    const QCString &height,
                                    bool hasCaption,
                                    const QCString &srcFile,
-                                   int srcLine
+                                   int srcLine, bool newFile
                                   )
 {
-  QCString baseName=makeBaseName(fileName);
+  QCString baseName=makeBaseName(fileName,".dot");
   baseName.prepend("dot_");
   QCString outDir = Config_getString(LATEX_OUTPUT);
-  QCString name = fileName;
-  writeDotGraphFromFile(name,outDir,baseName,GraphOutputFormat::EPS,srcFile,srcLine);
+  if (newFile) writeDotGraphFromFile(fileName,outDir,baseName,GraphOutputFormat::EPS,srcFile,srcLine);
   visitPreStart(m_t,hasCaption, baseName, width, height);
 }
 
@@ -2029,14 +1937,14 @@ void LatexDocVisitor::startMscFile(const QCString &fileName,
                                    const QCString &height,
                                    bool hasCaption,
                                    const QCString &srcFile,
-                                   int srcLine
+                                   int srcLine, bool newFile
                                   )
 {
-  QCString baseName=makeBaseName(fileName);
+  QCString baseName=makeBaseName(fileName,".msc");
   baseName.prepend("msc_");
 
   QCString outDir = Config_getString(LATEX_OUTPUT);
-  writeMscGraphFromFile(fileName,outDir,baseName,MscOutputFormat::EPS,srcFile,srcLine);
+  if (newFile) writeMscGraphFromFile(fileName,outDir,baseName,MscOutputFormat::EPS,srcFile,srcLine);
   visitPreStart(m_t,hasCaption, baseName, width, height);
 }
 
@@ -2047,30 +1955,29 @@ void LatexDocVisitor::endMscFile(bool hasCaption)
 }
 
 
-void LatexDocVisitor::writeMscFile(const QCString &baseName, const DocVerbatim &s)
+void LatexDocVisitor::writeMscFile(const QCString &fileName, const DocVerbatim &s, bool newFile)
 {
-  QCString shortName = makeShortName(baseName);
+  QCString shortName=makeBaseName(fileName,".msc");
   QCString outDir = Config_getString(LATEX_OUTPUT);
-  writeMscGraphFromFile(baseName+".msc",outDir,shortName,MscOutputFormat::EPS,s.srcFile(),s.srcLine());
+  if (newFile) writeMscGraphFromFile(fileName,outDir,shortName,MscOutputFormat::EPS,s.srcFile(),s.srcLine());
   visitPreStart(m_t, s.hasCaption(), shortName, s.width(),s.height());
   visitCaption(s.children());
   visitPostEnd(m_t, s.hasCaption());
 }
 
-
 void LatexDocVisitor::startDiaFile(const QCString &fileName,
                                    const QCString &width,
                                    const QCString &height,
                                    bool hasCaption,
                                    const QCString &srcFile,
-                                   int srcLine
+                                   int srcLine, bool newFile
                                   )
 {
-  QCString baseName=makeBaseName(fileName);
+  QCString baseName=makeBaseName(fileName,".dia");
   baseName.prepend("dia_");
 
   QCString outDir = Config_getString(LATEX_OUTPUT);
-  writeDiaGraphFromFile(fileName,outDir,baseName,DiaOutputFormat::EPS,srcFile,srcLine);
+  if (newFile) writeDiaGraphFromFile(fileName,outDir,baseName,DiaOutputFormat::EPS,srcFile,srcLine);
   visitPreStart(m_t,hasCaption, baseName, width, height);
 }
 
@@ -2080,20 +1987,9 @@ void LatexDocVisitor::endDiaFile(bool hasCaption)
   visitPostEnd(m_t,hasCaption);
 }
 
-
-void LatexDocVisitor::writeDiaFile(const QCString &baseName, const DocVerbatim &s)
-{
-  QCString shortName = makeShortName(baseName);
-  QCString outDir = Config_getString(LATEX_OUTPUT);
-  writeDiaGraphFromFile(baseName+".dia",outDir,shortName,DiaOutputFormat::EPS,s.srcFile(),s.srcLine());
-  visitPreStart(m_t, s.hasCaption(), shortName, s.width(), s.height());
-  visitCaption(s.children());
-  visitPostEnd(m_t, s.hasCaption());
-}
-
 void LatexDocVisitor::writePlantUMLFile(const QCString &baseName, const DocVerbatim &s)
 {
-  QCString shortName = makeShortName(baseName);
+  QCString shortName = stripPath(baseName);
   if (s.useBitmap())
   {
     if (shortName.find('.')==-1) shortName += ".png";
@@ -2119,19 +2015,25 @@ void LatexDocVisitor::startPlantUmlFile(const QCString &fileName,
   readInputFile(fileName,inBuf);
 
   bool useBitmap = inBuf.find("@startditaa") != std::string::npos;
-  QCString baseName = PlantumlManager::instance().writePlantUMLSource(
-                              outDir,QCString(),inBuf.c_str(),
+  auto baseNameVector = PlantumlManager::instance().writePlantUMLSource(
+                              outDir,QCString(),inBuf,
                               useBitmap ? PlantumlManager::PUML_BITMAP : PlantumlManager::PUML_EPS,
                               QCString(),srcFile,srcLine,false);
-  baseName=makeBaseName(baseName);
-  QCString shortName = makeShortName(baseName);
-  if (useBitmap)
+  bool first = true;
+  for (const auto &bName: baseNameVector)
   {
-    if (shortName.find('.')==-1) shortName += ".png";
+    QCString baseName = makeBaseName(bName,".pu");
+    QCString shortName = stripPath(baseName);
+    if (useBitmap)
+    {
+      if (shortName.find('.')==-1) shortName += ".png";
+    }
+    PlantumlManager::instance().generatePlantUMLOutput(baseName,outDir,
+                                useBitmap ? PlantumlManager::PUML_BITMAP : PlantumlManager::PUML_EPS);
+    if (!first) endPlantUmlFile(hasCaption);
+    first = false;
+    visitPreStart(m_t,hasCaption, shortName, width, height);
   }
-  PlantumlManager::instance().generatePlantUMLOutput(baseName,outDir,
-                              useBitmap ? PlantumlManager::PUML_BITMAP : PlantumlManager::PUML_EPS);
-  visitPreStart(m_t,hasCaption, shortName, width, height);
 }
 
 void LatexDocVisitor::endPlantUmlFile(bool hasCaption)
