@@ -209,9 +209,10 @@ void ManDocVisitor::operator()(const DocVerbatim &s)
       m_t << ".PP\n";
       m_t << ".nf\n";
       getCodeParser(lang).parseCode(m_ci,s.context(),s.text(),
-                                        langExt,
-                                        Config_getBool(STRIP_CODE_COMMENTS),
-                                        s.isExample(),s.exampleFile());
+                                    langExt,
+                                    Config_getBool(STRIP_CODE_COMMENTS),
+                                    CodeParserOptions().setExample(s.isExample(),s.exampleFile())
+                                   );
       if (!m_firstCol) m_t << "\n";
       m_t << ".fi\n";
       m_t << ".PP\n";
@@ -270,18 +271,14 @@ void ManDocVisitor::operator()(const DocInclude &inc)
          FileInfo cfi( inc.file().str() );
          auto fd = createFileDef( cfi.dirPath(), cfi.fileName() );
          getCodeParser(inc.extension()).parseCode(m_ci,inc.context(),
-                                           inc.text(),
-                                           langExt,
-                                           inc.stripCodeComments(),
-                                           inc.isExample(),
-                                           inc.exampleFile(),
-                                           fd.get(), // fileDef,
-                                           -1,    // start line
-                                           -1,    // end line
-                                           FALSE, // inline fragment
-                                           nullptr,     // memberDef
-                                           TRUE
-					   );
+                                                  inc.text(),
+                                                  langExt,
+                                                  inc.stripCodeComments(),
+                                                  CodeParserOptions()
+                                                  .setExample(inc.isExample(),inc.exampleFile())
+                                                  .setFileDef(fd.get())
+                                                  .setInlineFragment(true)
+                                                 );
          m_t << ".fi\n";
          m_t << ".PP\n";
          m_firstCol=TRUE;
@@ -292,18 +289,14 @@ void ManDocVisitor::operator()(const DocInclude &inc)
       m_t << ".PP\n";
       m_t << ".nf\n";
       getCodeParser(inc.extension()).parseCode(m_ci,inc.context(),
-                                        inc.text(),
-                                        langExt,
-                                        inc.stripCodeComments(),
-                                        inc.isExample(),
-                                        inc.exampleFile(),
-                                        nullptr,     // fileDef
-                                        -1,    // startLine
-                                        -1,    // endLine
-                                        TRUE,  // inlineFragment
-                                        nullptr,     // memberDef
-                                        FALSE
-				       );
+                                               inc.text(),
+                                               langExt,
+                                               inc.stripCodeComments(),
+                                               CodeParserOptions()
+                                               .setExample(inc.isExample(),inc.exampleFile())
+                                               .setInlineFragment(true)
+                                               .setShowLineNumbers(false)
+				              );
       m_t << ".fi\n";
       m_t << ".PP\n";
       m_firstCol=TRUE;
@@ -335,13 +328,13 @@ void ManDocVisitor::operator()(const DocInclude &inc)
       m_t << ".PP\n";
       m_t << ".nf\n";
       CodeFragmentManager::instance().parseCodeFragment(m_ci,
-                                          inc.file(),
-                                          inc.blockId(),
-                                          inc.context(),
-                                          inc.type()==DocInclude::SnippetWithLines,
-                                          inc.trimLeft(),
-                                          inc.stripCodeComments()
-                                         );
+                                                        inc.file(),
+                                                        inc.blockId(),
+                                                        inc.context(),
+                                                        inc.type()==DocInclude::SnippetWithLines,
+                                                        inc.trimLeft(),
+                                                        inc.stripCodeComments()
+                                                       );
       m_t << ".fi\n";
       m_t << ".PP\n";
       m_firstCol=TRUE;
@@ -380,15 +373,12 @@ void ManDocVisitor::operator()(const DocIncOperator &op)
       }
 
       getCodeParser(locLangExt).parseCode(m_ci,op.context(),op.text(),langExt,
-                                        op.stripCodeComments(),
-                                        op.isExample(),op.exampleFile(),
-                                        fd.get(),     // fileDef
-                                        op.line(),    // startLine
-                                        -1,    // endLine
-                                        FALSE, // inline fragment
-                                        nullptr,     // memberDef
-                                        op.showLineNo()  // show line numbers
-                                       );
+                                          op.stripCodeComments(),
+                                          CodeParserOptions()
+                                          .setExample(op.isExample(),op.exampleFile())
+                                          .setFileDef(fd.get())
+                                          .setShowLineNumbers(op.showLineNo())
+                                         );
     }
     pushHidden(m_hide);
     m_hide=TRUE;
