@@ -17,6 +17,7 @@
 #include <cstdlib>
 #include <mutex>
 #include <atomic>
+#include <unordered_set>
 
 #include "config.h"
 #include "debug.h"
@@ -25,6 +26,7 @@
 #include "doxygen.h"
 #include "fileinfo.h"
 #include "dir.h"
+#include "md5.h"
 
 // globals
 static QCString        g_warnFormat;
@@ -37,13 +39,206 @@ static QCString        g_warnlogFile;
 static bool            g_warnlogTemp = false;
 static std::atomic_bool g_warnStat = false;
 static std::mutex      g_mutex;
+static std::unordered_set<std::string> g_warnHash;
+
+//-----------------------------------------------------------------------------------------
+
+static bool checkWarnMessage(QCString result)
+{
+  uint8_t md5_sig[16];
+  char sigStr[33];
+  MD5Buffer(result.data(),result.length(),md5_sig);
+  MD5SigToString(md5_sig,sigStr);
+
+  return g_warnHash.insert(sigStr).second;
+}
+
+static void format_warn(const QCString &file,int line,const QCString &text)
+{
+  QCString fileSubst = file.isEmpty() ? "<unknown>" : file;
+  QCString lineSubst; lineSubst.setNum(line);
+  QCString versionSubst;
+  // substitute markers by actual values
+  QCString msgText =
+      substitute(
+        substitute(
+          substitute(
+            substitute(
+              g_warnFormat,
+              "$file",fileSubst
+            ),
+            "$line",lineSubst
+          ),
+          "$version",versionSubst
+        ),
+        "$text",text
+      );
+  if (g_warnBehavior == WARN_AS_ERROR_t::YES)
+  {
+    msgText += " (warning treated as error, aborting now)";
+  }
+  msgText += '\n';
+
+  {
+    std::unique_lock<std::mutex> lock(g_mutex);
+    // print resulting message
+    if (checkWarnMessage(msgText)) fwrite(msgText.data(),1,msgText.length(),g_warnFile);
+  }
+  if (g_warnBehavior == WARN_AS_ERROR_t::YES)
+  {
+    if (g_warnFile != stderr && !Config_getBool(QUIET))
+    {
+      msg("See '{}' for the reason of termination.\n",g_warnlogFile);
+    }
+    exit(1);
+  }
+  g_warnStat = true;
+}
+
+//-----------------------------------------------------------------------------------------
+
+static void handle_warn_as_error()
+{
+  if (g_warnBehavior == WARN_AS_ERROR_t::YES)
+  {
+    {
+      std::unique_lock<std::mutex> lock(g_mutex);
+      QCString msgText = " (warning treated as error, aborting now)\n";
+      fwrite(msgText.data(),1,msgText.length(),g_warnFile);
+      if (g_warnFile != stderr && !Config_getBool(QUIET))
+      {
+        // cannot use `msg` due to the mutex
+        fprintf(stdout,"See '%s' for the reason of termination.\n",qPrint(g_warnlogFile));
+      }
+    }
+    exit(1);
+  }
+  g_warnStat = true;
+}
+
+//-----------------------------------------------------------------------------------------
+
+static void do_warn(const QCString &file, int line, const char *prefix, fmt::string_view fmt, fmt::format_args args)
+{
+  format_warn(file,line,prefix+fmt::vformat(fmt,args));
+  handle_warn_as_error();
+}
+
+//-----------------------------------------------------------------------------------------
+
+void msg_(fmt::string_view fmt, fmt::format_args args)
+{
+  if (!Config_getBool(QUIET))
+  {
+    std::unique_lock<std::mutex> lock(g_mutex);
+    if (Debug::isFlagSet(Debug::Time))
+    {
+      fmt::print("{:.3f} sec: ",(static_cast<double>(Debug::elapsedTime())));
+    }
+    fmt::print("{}",fmt::vformat(fmt,args));
+  }
+}
+
+//-----------------------------------------------------------------------------------------
+
+void warn_(WarningType type, const QCString &file, int line, fmt::string_view fmt, fmt::format_args args)
+{
+  bool enabled = false;
+  switch (type)
+  {
+    case WarningType::Generic:       enabled = Config_getBool(WARNINGS);               break;
+    case WarningType::Undocumented:  enabled = Config_getBool(WARN_IF_UNDOCUMENTED);   break;
+    case WarningType::IncompleteDoc: enabled = Config_getBool(WARN_IF_INCOMPLETE_DOC); break;
+    case WarningType::DocError:      enabled = Config_getBool(WARN_IF_DOC_ERROR);      break;
+    case WarningType::Layout:        enabled = Config_getBool(WARN_LAYOUT_FILE);       break;
+  }
+  if (enabled)
+  {
+    do_warn(file, line, g_warningStr, fmt, args);
+  }
+}
+
+//-----------------------------------------------------------------------------------------
+
+void warn_uncond_(fmt::string_view fmt, fmt::format_args args)
+{
+  {
+    std::unique_lock<std::mutex> lock(g_mutex);
+    if (checkWarnMessage(g_errorStr+fmt::vformat(fmt,args))) fmt::print(g_warnFile,"{}{}",g_warningStr,vformat(fmt,args));
+  }
+  handle_warn_as_error();
+}
+
+//-----------------------------------------------------------------------------------------
+
+void err_(fmt::string_view fmt, fmt::format_args args)
+{
+  {
+    std::unique_lock<std::mutex> lock(g_mutex);
+    if (checkWarnMessage(g_errorStr+fmt::vformat(fmt,args))) fmt::print(g_warnFile,"{}{}",g_errorStr,fmt::vformat(fmt,args));
+  }
+  handle_warn_as_error();
+}
+
+//-----------------------------------------------------------------------------------------
+
+void err_full_(const QCString &file, int line, fmt::string_view fmt, fmt::format_args args)
+{
+  format_warn(file,line,g_errorStr+fmt::vformat(fmt,args));
+}
+
+//-----------------------------------------------------------------------------------------
+
+void term_(fmt::string_view fmt, fmt::format_args args)
+{
+  {
+    std::unique_lock<std::mutex> lock(g_mutex);
+    if (checkWarnMessage(g_errorStr+fmt::vformat(fmt,args))) fmt::print(g_warnFile, "{}{}", g_errorStr, fmt::vformat(fmt,args));
+    if (g_warnFile != stderr)
+    {
+      size_t l = strlen(g_errorStr);
+      for (size_t i=0; i<l; i++) fmt::print(g_warnFile, " ");
+      fmt::print(g_warnFile, "{}\n", "Exiting...");
+      if (!Config_getBool(QUIET))
+      {
+        // cannot use `msg` due to the mutex
+        fmt::print("See '{}' for the reason of termination.\n",g_warnlogFile);
+      }
+    }
+  }
+  exit(1);
+}
+
+//-----------------------------------------------------------------------------------------
+
+QCString warn_line(const QCString &file,int line)
+{
+  QCString fileSubst = file.isEmpty() ? "<unknown>" : file;
+  QCString lineSubst; lineSubst.setNum(line);
+  return  substitute(
+            substitute(
+              g_warnLineFormat,
+              "$file",fileSubst
+            ),
+            "$line",lineSubst
+          );
+}
+
+//-----------------------------------------------------------------------------------------
+
+void warn_flush()
+{
+  fflush(g_warnFile);
+}
+
+//-----------------------------------------------------------------------------------------
 
 void initWarningFormat()
 {
-  g_warnFormat = Config_getString(WARN_FORMAT);
+  g_warnFormat     = Config_getString(WARN_FORMAT);
   g_warnLineFormat = Config_getString(WARN_LINE_FORMAT);
-  g_warnBehavior = Config_getEnum(WARN_AS_ERROR);
-  g_warnlogFile = Config_getString(WARN_LOGFILE);
+  g_warnBehavior   = Config_getEnum(WARN_AS_ERROR);
+  g_warnlogFile    = Config_getString(WARN_LOGFILE);
   if (g_warnlogFile.isEmpty() && g_warnBehavior == WARN_AS_ERROR_t::FAIL_ON_WARNINGS_PRINT)
   {
     uint32_t pid = Portable::pid();
@@ -60,18 +255,18 @@ void initWarningFormat()
     else
     {
       FileInfo fi(g_warnlogFile.str());
-      Dir d(fi.dirPath().c_str());
-      if (!d.exists() && !d.mkdir(fi.dirPath().c_str()))
+      Dir d(fi.dirPath());
+      if (!d.exists() && !d.mkdir(fi.dirPath()))
       {
         // point it to something valid, because warn() relies on it
         g_warnFile = stderr;
-        err("Cannot create directory for '%s', redirecting 'WARN_LOGFILE' output to 'stderr'\n",g_warnlogFile.data());
+        err("Cannot create directory for '{}', redirecting 'WARN_LOGFILE' output to 'stderr'\n",g_warnlogFile);
       }
       else if (!(g_warnFile = Portable::fopen(g_warnlogFile,"w")))
       {
         // point it to something valid, because warn() relies on it
         g_warnFile = stderr;
-        err("Cannot open '%s' for writing, redirecting 'WARN_LOGFILE' output to 'stderr'\n",g_warnlogFile.data());
+        err("Cannot open '{}' for writing, redirecting 'WARN_LOGFILE' output to 'stderr'\n",g_warnlogFile);
       }
     }
   }
@@ -94,209 +289,9 @@ void initWarningFormat()
   });
 }
 
+//-----------------------------------------------------------------------------------------
 
-void msg(const char *fmt, ...)
-{
-  if (!Config_getBool(QUIET))
-  {
-    std::unique_lock<std::mutex> lock(g_mutex);
-    if (Debug::isFlagSet(Debug::Time))
-    {
-      printf("%.3f sec: ",(static_cast<double>(Debug::elapsedTime())));
-    }
-    va_list args;
-    va_start(args, fmt);
-    vfprintf(stdout, fmt, args);
-    va_end(args);
-  }
-}
-
-static void format_warn(const QCString &file,int line,const QCString &text)
-{
-  QCString fileSubst = file.isEmpty() ? "<unknown>" : file;
-  QCString lineSubst; lineSubst.setNum(line);
-  QCString textSubst = text;
-  QCString versionSubst;
-  // substitute markers by actual values
-  QCString msgText =
-      substitute(
-        substitute(
-          substitute(
-            substitute(
-              g_warnFormat,
-              "$file",fileSubst
-            ),
-            "$line",lineSubst
-          ),
-          "$version",versionSubst
-        ),
-        "$text",textSubst
-      );
-  if (g_warnBehavior == WARN_AS_ERROR_t::YES)
-  {
-    msgText += " (warning treated as error, aborting now)";
-  }
-  msgText += '\n';
-
-  {
-    std::unique_lock<std::mutex> lock(g_mutex);
-    // print resulting message
-    fwrite(msgText.data(),1,msgText.length(),g_warnFile);
-  }
-  if (g_warnBehavior == WARN_AS_ERROR_t::YES)
-  {
-    Doxygen::terminating=true;
-    exit(1);
-  }
-  g_warnStat = true;
-}
-
-static void handle_warn_as_error()
-{
-  if (g_warnBehavior == WARN_AS_ERROR_t::YES)
-  {
-    {
-      std::unique_lock<std::mutex> lock(g_mutex);
-      QCString msgText = " (warning treated as error, aborting now)\n";
-      fwrite(msgText.data(),1,msgText.length(),g_warnFile);
-    }
-    Doxygen::terminating=true;
-    exit(1);
-  }
-  g_warnStat = true;
-}
-
-static void do_warn(bool enabled, const QCString &file, int line, const char *prefix, const char *fmt, va_list args)
-{
-  if (!enabled) return; // warning type disabled
-
-  va_list argsCopy;
-  va_copy(argsCopy, args);
-
-  size_t l=0;
-  if (prefix)
-  {
-    l=strlen(prefix);
-  }
-  // determine needed buffersize based on:
-  // format + arguments
-  // prefix
-  // 1 position for `\0`
-  size_t bufSize = vsnprintf(nullptr, 0, fmt, args) + l;
-  QCString text(bufSize, QCString::ExplicitSize);
-  if (prefix)
-  {
-    qstrncpy(text.rawData(),prefix,bufSize);
-  }
-  vsnprintf(text.rawData()+l, bufSize-l+1, fmt, argsCopy);
-  text[bufSize]='\0';
-  format_warn(file,line,text);
-
-  va_end(argsCopy);
-}
-
-QCString warn_line(const QCString &file,int line)
-{
-  QCString fileSubst = file.isEmpty() ? "<unknown>" : file;
-  QCString lineSubst; lineSubst.setNum(line);
-  return  substitute(
-            substitute(
-              g_warnLineFormat,
-              "$file",fileSubst
-            ),
-            "$line",lineSubst
-          );
-}
-void warn_(const QCString &file,int line,const char *fmt, ...)
-{
-  va_list args;
-  va_start(args, fmt);
-  do_warn(Config_getBool(WARNINGS), file, line, g_warningStr, fmt, args);
-  va_end(args);
-}
-
-void va_warn(const QCString &file,int line,const char *fmt,va_list args)
-{
-  do_warn(Config_getBool(WARNINGS), file, line, g_warningStr, fmt, args);
-}
-
-void warn_undoc_(const QCString &file,int line,const char *fmt, ...)
-{
-  va_list args;
-  va_start(args, fmt);
-  do_warn(Config_getBool(WARN_IF_UNDOCUMENTED), file, line, g_warningStr, fmt, args);
-  va_end(args);
-}
-
-void warn_incomplete_doc_(const QCString &file,int line,const char *fmt, ...)
-{
-  va_list args;
-  va_start(args, fmt);
-  do_warn(Config_getBool(WARN_IF_INCOMPLETE_DOC), file, line, g_warningStr, fmt, args);
-  va_end(args);
-}
-
-void warn_doc_error_(const QCString &file,int line,const char *fmt, ...)
-{
-  va_list args;
-  va_start(args, fmt);
-  do_warn(Config_getBool(WARN_IF_DOC_ERROR), file, line, g_warningStr, fmt, args);
-  va_end(args);
-}
-
-void warn_uncond_(const char *fmt, ...)
-{
-  va_list args;
-  va_start(args, fmt);
-  vfprintf(g_warnFile, (QCString(g_warningStr) + fmt).data(), args);
-  va_end(args);
-  handle_warn_as_error();
-}
-
-void err_(const char *fmt, ...)
-{
-  va_list args;
-  va_start(args, fmt);
-  vfprintf(g_warnFile, (QCString(g_errorStr) + fmt).data(), args);
-  va_end(args);
-  handle_warn_as_error();
-}
-
-extern void err_full_(const QCString &file,int line,const char *fmt, ...)
-{
-  va_list args;
-  va_start(args, fmt);
-  do_warn(TRUE, file, line, g_errorStr, fmt, args);
-  va_end(args);
-}
-
-void term_(const char *fmt, ...)
-{
-  {
-    std::unique_lock<std::mutex> lock(g_mutex);
-    va_list args;
-    va_start(args, fmt);
-    vfprintf(g_warnFile, (QCString(g_errorStr) + fmt).data(), args);
-    va_end(args);
-    if (g_warnFile != stderr)
-    {
-      size_t l = strlen(g_errorStr);
-      for (size_t i=0; i<l; i++) fprintf(g_warnFile, " ");
-      fprintf(g_warnFile, "%s\n", "Exiting...");
-    }
-  }
-  Doxygen::terminating=true;
-  exit(1);
-}
-
-void warn_flush()
-{
-  fflush(g_warnFile);
-}
-
-
-
-extern void finishWarnExit()
+void finishWarnExit()
 {
   fflush(stdout);
   if (g_warnBehavior == WARN_AS_ERROR_t::FAIL_ON_WARNINGS_PRINT && g_warnlogFile != "-")
@@ -311,14 +306,14 @@ extern void finishWarnExit()
     if (!warnFile.is_open())
     {
       g_warnFile = stderr;
-      err("Cannot open warnings file '%s' for reading\n",g_warnlogFile.data());
+      err("Cannot open warnings file '{}' for reading\n",g_warnlogFile);
     }
     else
     {
       std::string line;
       while (getline(warnFile,line))
       {
-        fprintf(stderr,"%s\n",line.c_str());
+        fmt::print(stderr,"{}\n",line);
       }
       warnFile.close();
     }
@@ -329,7 +324,6 @@ extern void finishWarnExit()
   if (g_warnStat && (g_warnBehavior == WARN_AS_ERROR_t::FAIL_ON_WARNINGS ||
                      g_warnBehavior == WARN_AS_ERROR_t::FAIL_ON_WARNINGS_PRINT))
   {
-    Doxygen::terminating=true;
     exit(1);
   }
 }
