@@ -1,6 +1,4 @@
 /******************************************************************************
- *
- *
  *
  * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
@@ -102,17 +100,18 @@ class NamespaceDefImpl : public DefinitionMixin<NamespaceDefMutable>
     const Definition *findInnerCompound(const QCString &name) const override;
     void addInnerCompound(Definition *d) override;
     void addListReferences() override;
+    void addRequirementReferences() override;
     void setFileName(const QCString &fn) override;
     bool subGrouping() const override { return m_subGrouping; }
     MemberList *getMemberList(MemberListType lt) const override;
     const MemberLists &getMemberLists() const override { return m_memberLists; }
     const MemberDef *getMemberByName(const QCString &) const override;
     const MemberGroupList &getMemberGroups() const override { return m_memberGroups; }
-    ClassLinkedRefMap getClasses() const override { return classes; }
-    ClassLinkedRefMap getInterfaces() const override { return interfaces; }
-    ClassLinkedRefMap getStructs() const override { return structs; }
-    ClassLinkedRefMap getExceptions() const override { return exceptions; }
-    NamespaceLinkedRefMap getNamespaces() const override { return namespaces; }
+    ClassLinkedRefMap getClasses() const override { return m_classes; }
+    ClassLinkedRefMap getInterfaces() const override { return m_interfaces; }
+    ClassLinkedRefMap getStructs() const override { return m_structs; }
+    ClassLinkedRefMap getExceptions() const override { return m_exceptions; }
+    NamespaceLinkedRefMap getNamespaces() const override { return m_namespaces; }
     ConceptLinkedRefMap getConcepts() const override { return m_concepts; }
     void setName(const QCString &name) override;
 
@@ -158,12 +157,12 @@ class NamespaceDefImpl : public DefinitionMixin<NamespaceDefMutable>
     MemberLinkedRefMap    m_allMembers;
     MemberLists           m_memberLists;
     MemberGroupList       m_memberGroups;
-    ClassLinkedRefMap     classes;
-    ClassLinkedRefMap     interfaces;
-    ClassLinkedRefMap     structs;
-    ClassLinkedRefMap     exceptions;
+    ClassLinkedRefMap     m_classes;
+    ClassLinkedRefMap     m_interfaces;
+    ClassLinkedRefMap     m_structs;
+    ClassLinkedRefMap     m_exceptions;
     ConceptLinkedRefMap   m_concepts;
-    NamespaceLinkedRefMap namespaces;
+    NamespaceLinkedRefMap m_namespaces;
     bool                  m_subGrouping = false;
     enum { NAMESPACE, MODULE, CONSTANT_GROUP, LIBRARY } m_type;
     bool m_isPublished = false;
@@ -409,21 +408,21 @@ void NamespaceDefImpl::addInnerCompound(Definition *d)
 
 void NamespaceDefImpl::insertClass(ClassDef *cd)
 {
-  ClassLinkedRefMap *d = &classes;
+  ClassLinkedRefMap *d = &m_classes;
 
   if (Config_getBool(OPTIMIZE_OUTPUT_SLICE))
   {
     if (cd->compoundType()==ClassDef::Interface)
     {
-      d = &interfaces;
+      d = &m_interfaces;
     }
     else if (cd->compoundType()==ClassDef::Struct)
     {
-      d = &structs;
+      d = &m_structs;
     }
     else if (cd->compoundType()==ClassDef::Exception)
     {
-      d = &exceptions;
+      d = &m_exceptions;
     }
   }
 
@@ -437,7 +436,7 @@ void NamespaceDefImpl::insertConcept(ConceptDef *cd)
 
 void NamespaceDefImpl::insertNamespace(NamespaceDef *nd)
 {
-  namespaces.add(nd->name(),nd);
+  m_namespaces.add(nd->name(),nd);
 }
 
 
@@ -577,8 +576,7 @@ void NamespaceDefImpl::insertMember(MemberDef *md)
         }
         if (aliasMd)
         {
-          QCString name = md->name();
-          MemberName *mn = Doxygen::functionNameLinkedMap->add(name);
+          MemberName *mn = Doxygen::functionNameLinkedMap->add(md->name());
           mn->push_back(std::move(aliasMd));
         }
       }
@@ -617,7 +615,7 @@ void NamespaceDefImpl::writeTagFile(TextStream &tagFile)
     {
       case LayoutDocEntry::NamespaceNestedNamespaces:
         {
-          for (const auto &nd : namespaces)
+          for (const auto &nd : m_namespaces)
           {
             if (nd->isLinkableInProject())
             {
@@ -628,22 +626,22 @@ void NamespaceDefImpl::writeTagFile(TextStream &tagFile)
         break;
       case LayoutDocEntry::NamespaceClasses:
         {
-          writeClassesToTagFile(tagFile, classes);
+          writeClassesToTagFile(tagFile, m_classes);
         }
         break;
       case LayoutDocEntry::NamespaceInterfaces:
         {
-          writeClassesToTagFile(tagFile, interfaces);
+          writeClassesToTagFile(tagFile, m_interfaces);
         }
         break;
       case LayoutDocEntry::NamespaceStructs:
         {
-          writeClassesToTagFile(tagFile, structs);
+          writeClassesToTagFile(tagFile, m_structs);
         }
         break;
       case LayoutDocEntry::NamespaceExceptions:
         {
-          writeClassesToTagFile(tagFile, exceptions);
+          writeClassesToTagFile(tagFile, m_exceptions);
         }
         break;
       case LayoutDocEntry::NamespaceConcepts:
@@ -699,8 +697,12 @@ void NamespaceDefImpl::writeDetailedDescription(OutputList &ol,const QCString &t
     ol.startTextBlock();
     if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF))
     {
-      ol.generateDoc(briefFile(),briefLine(),this,nullptr,briefDescription(),FALSE,FALSE,
-                     QCString(),FALSE,FALSE);
+      ol.generateDoc(briefFile(),
+                     briefLine(),
+                     this,
+                     nullptr,
+                     briefDescription(),
+                     DocOptions());
     }
     if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF) &&
         !documentation().isEmpty())
@@ -716,9 +718,15 @@ void NamespaceDefImpl::writeDetailedDescription(OutputList &ol,const QCString &t
     }
     if (!documentation().isEmpty())
     {
-      ol.generateDoc(docFile(),docLine(),this,nullptr,documentation()+"\n",TRUE,FALSE,
-                     QCString(),FALSE,FALSE);
+      ol.generateDoc(docFile(),
+                     docLine(),
+                     this,
+                     nullptr,
+                     documentation()+"\n",
+                     DocOptions()
+                     .setIndexWords(true));
     }
+    if (hasRequirementRefs()) writeRequirementRefs(ol);
     ol.endTextBlock();
   }
 }
@@ -729,9 +737,15 @@ void NamespaceDefImpl::writeBriefDescription(OutputList &ol)
   {
     auto parser { createDocParser() };
     auto ast    { validatingParseDoc(*parser.get(),
-                                     briefFile(),briefLine(),this,nullptr,
-                                     briefDescription(),TRUE,FALSE,
-                                     QCString(),TRUE,FALSE) };
+                                     briefFile(),
+                                     briefLine(),
+                                     this,
+                                     nullptr,
+                                     briefDescription(),
+                                     DocOptions()
+                                     .setIndexWords(true)
+                                     .setSingleLine(true))
+                 };
     if (!ast->isEmpty())
     {
       ol.startParagraph();
@@ -817,13 +831,13 @@ void NamespaceDefImpl::writeConcepts(OutputList &ol,const QCString &title)
 
 void NamespaceDefImpl::writeInlineClasses(OutputList &ol)
 {
-  classes.writeDocumentation(ol,this);
+  m_classes.writeDocumentation(ol,this);
 }
 
 void NamespaceDefImpl::writeNamespaceDeclarations(OutputList &ol,const QCString &title,
             bool const isConstantGroup)
 {
-  namespaces.writeDeclaration(ol,title,isConstantGroup,TRUE);
+  m_namespaces.writeDeclaration(ol,title,isConstantGroup,TRUE);
 }
 
 void NamespaceDefImpl::writeMemberGroups(OutputList &ol)
@@ -859,37 +873,37 @@ void NamespaceDefImpl::writeSummaryLinks(OutputList &ol) const
   for (const auto &lde : LayoutDocManager::instance().docEntries(LayoutDocManager::Namespace))
   {
     const LayoutDocEntrySection *ls = dynamic_cast<const LayoutDocEntrySection*>(lde.get());
-    if (lde->kind()==LayoutDocEntry::NamespaceClasses && classes.declVisible() && ls)
+    if (lde->kind()==LayoutDocEntry::NamespaceClasses && m_classes.declVisible() && ls)
     {
       QCString label = "nested-classes";
       ol.writeSummaryLink(QCString(),label,ls->title(lang),first);
       first=FALSE;
     }
-    else if (lde->kind()==LayoutDocEntry::NamespaceInterfaces && interfaces.declVisible() && ls)
+    else if (lde->kind()==LayoutDocEntry::NamespaceInterfaces && m_interfaces.declVisible() && ls)
     {
       QCString label = "interfaces";
       ol.writeSummaryLink(QCString(),label,ls->title(lang),first);
       first=FALSE;
     }
-    else if (lde->kind()==LayoutDocEntry::NamespaceStructs && structs.declVisible() && ls)
+    else if (lde->kind()==LayoutDocEntry::NamespaceStructs && m_structs.declVisible() && ls)
     {
       QCString label = "structs";
       ol.writeSummaryLink(QCString(),label,ls->title(lang),first);
       first=FALSE;
     }
-    else if (lde->kind()==LayoutDocEntry::NamespaceExceptions && exceptions.declVisible() && ls)
+    else if (lde->kind()==LayoutDocEntry::NamespaceExceptions && m_exceptions.declVisible() && ls)
     {
       QCString label = "exceptions";
       ol.writeSummaryLink(QCString(),label,ls->title(lang),first);
       first=FALSE;
     }
-    else if (lde->kind()==LayoutDocEntry::NamespaceNestedNamespaces && namespaces.declVisible(false) && ls)
+    else if (lde->kind()==LayoutDocEntry::NamespaceNestedNamespaces && m_namespaces.declVisible(false) && ls)
     {
       QCString label = "namespaces";
       ol.writeSummaryLink(QCString(),label,ls->title(lang),first);
       first=FALSE;
     }
-    else if (lde->kind()==LayoutDocEntry::NamespaceNestedConstantGroups && namespaces.declVisible(true) && ls)
+    else if (lde->kind()==LayoutDocEntry::NamespaceNestedConstantGroups && m_namespaces.declVisible(true) && ls)
     {
       QCString label = "constantgroups";
       ol.writeSummaryLink(QCString(),label,ls->title(lang),first);
@@ -980,7 +994,7 @@ void NamespaceDefImpl::writeDocumentation(OutputList &ol)
   //bool fortranOpt = Config_getBool(OPTIMIZE_FOR_FORTRAN);
 
   QCString pageTitle = title();
-  startFile(ol,getOutputFileBase(),name(),pageTitle,HighlightedItem::NamespaceVisible,!generateTreeView);
+  startFile(ol,getOutputFileBase(),false,name(),pageTitle,HighlightedItem::NamespaceVisible,!generateTreeView);
 
   if (!generateTreeView)
   {
@@ -1014,22 +1028,22 @@ void NamespaceDefImpl::writeDocumentation(OutputList &ol)
         break;
       case LayoutDocEntry::NamespaceClasses:
         {
-          if (ls) writeClassDeclarations(ol,ls->title(lang),classes);
+          if (ls) writeClassDeclarations(ol,ls->title(lang),m_classes);
         }
         break;
       case LayoutDocEntry::NamespaceInterfaces:
         {
-          if (ls) writeClassDeclarations(ol,ls->title(lang),interfaces);
+          if (ls) writeClassDeclarations(ol,ls->title(lang),m_interfaces);
         }
         break;
       case LayoutDocEntry::NamespaceStructs:
         {
-          if (ls) writeClassDeclarations(ol,ls->title(lang),structs);
+          if (ls) writeClassDeclarations(ol,ls->title(lang),m_structs);
         }
         break;
       case LayoutDocEntry::NamespaceExceptions:
         {
-          if (ls) writeClassDeclarations(ol,ls->title(lang),exceptions);
+          if (ls) writeClassDeclarations(ol,ls->title(lang),m_exceptions);
         }
         break;
       case LayoutDocEntry::NamespaceConcepts:
@@ -1253,19 +1267,15 @@ const Definition *NamespaceDefImpl::findInnerCompound(const QCString &n) const
 
 void NamespaceDefImpl::addListReferences()
 {
-  //bool fortranOpt = Config_getBool(OPTIMIZE_FOR_FORTRAN);
-  {
-    const RefItemVector &xrefItems = xrefListItems();
-    addRefItem(xrefItems,
-        qualifiedName(),
-        getLanguage()==SrcLangExt::Fortran ?
-          theTranslator->trModule(TRUE,TRUE) :
-          theTranslator->trNamespace(TRUE,TRUE),
-        getOutputFileBase(),displayName(),
-        QCString(),
-        this
-        );
-  }
+  addRefItem(xrefListItems(),
+             qualifiedName(),
+             getLanguage()==SrcLangExt::Fortran ?
+               theTranslator->trModule(TRUE,TRUE) :
+               theTranslator->trNamespace(TRUE,TRUE),
+             getOutputFileBase(),displayName(),
+             QCString(),
+             this
+            );
   for (const auto &mg : m_memberGroups)
   {
     mg->addListReferences(this);
@@ -1279,6 +1289,22 @@ void NamespaceDefImpl::addListReferences()
   }
 }
 
+void NamespaceDefImpl::addRequirementReferences()
+{
+  RequirementManager::instance().addRequirementRefsForSymbol(this);
+  for (const auto &mg : m_memberGroups)
+  {
+    mg->addRequirementReferences(this);
+  }
+  for (auto &ml : m_memberLists)
+  {
+    if (ml->listType().isDocumentation())
+    {
+      ml->addRequirementReferences(this);
+    }
+  }
+}
+
 QCString NamespaceDefImpl::displayName(bool includeScope) const
 {
   return makeDisplayName(this,includeScope);
@@ -1414,8 +1440,13 @@ void NamespaceLinkedRefMap::writeDeclaration(OutputList &ol,const QCString &titl
       if (!nd->briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
       {
         ol.startMemberDescription(nd->getOutputFileBase());
-        ol.generateDoc(nd->briefFile(),nd->briefLine(),nd,nullptr,nd->briefDescription(),FALSE,FALSE,
-                       QCString(),TRUE,FALSE);
+        ol.generateDoc(nd->briefFile(),
+                       nd->briefLine(),
+                       nd,
+                       nullptr,
+                       nd->briefDescription(),
+                       DocOptions()
+                       .setSingleLine(true));
         ol.endMemberDescription();
       }
       ol.endMemberDeclaration(QCString(),QCString());
@@ -1453,25 +1484,35 @@ void NamespaceDefImpl::sortMemberLists()
     if (ml->needsSorting()) { ml->sort(); ml->setNeedsSorting(FALSE); }
   }
 
-  auto classComp = [](const ClassLinkedRefMap::Ptr &c1,const ClassLinkedRefMap::Ptr &c2)
+  if (Config_getBool(SORT_BRIEF_DOCS))
   {
-    return Config_getBool(SORT_BY_SCOPE_NAME)          ?
-      qstricmp_sort(c1->name(), c2->name())<0          :
-      qstricmp_sort(c1->className(), c2->className())<0;
-  };
+    auto classComp = [](const ClassLinkedRefMap::Ptr &c1,const ClassLinkedRefMap::Ptr &c2)
+    {
+      return Config_getBool(SORT_BY_SCOPE_NAME)          ?
+        qstricmp_sort(c1->name(), c2->name())<0          :
+        qstricmp_sort(c1->className(), c2->className())<0;
+    };
 
-  std::stable_sort(classes.begin(),   classes.end(),   classComp);
-  std::stable_sort(interfaces.begin(),interfaces.end(),classComp);
-  std::stable_sort(structs.begin(),   structs.end(),   classComp);
-  std::stable_sort(exceptions.begin(),exceptions.end(),classComp);
+    std::stable_sort(m_classes.begin(),   m_classes.end(),   classComp);
+    std::stable_sort(m_interfaces.begin(),m_interfaces.end(),classComp);
+    std::stable_sort(m_structs.begin(),   m_structs.end(),   classComp);
+    std::stable_sort(m_exceptions.begin(),m_exceptions.end(),classComp);
 
+    auto conceptComp = [](const ConceptLinkedRefMap::Ptr &c1,const ConceptLinkedRefMap::Ptr &c2)
+    {
+      return qstricmp_sort(c1->name(),c2->name())<0;
+    };
 
-  auto namespaceComp = [](const NamespaceLinkedRefMap::Ptr &n1,const NamespaceLinkedRefMap::Ptr &n2)
-  {
-    return qstricmp_sort(n1->name(),n2->name())<0;
-  };
+    std::stable_sort(m_concepts.begin(), m_concepts.end(), conceptComp);
+
+    auto namespaceComp = [](const NamespaceLinkedRefMap::Ptr &n1,const NamespaceLinkedRefMap::Ptr &n2)
+    {
+      return qstricmp_sort(n1->name(),n2->name())<0;
+    };
+
+    std::stable_sort(m_namespaces.begin(),m_namespaces.end(),namespaceComp);
+  }
 
-  std::stable_sort(namespaces.begin(),namespaces.end(),namespaceComp);
 }
 
 MemberList *NamespaceDefImpl::getMemberList(MemberListType lt) const
@@ -1766,7 +1807,7 @@ static NamespaceDef *getResolvedNamespaceRec(StringSet &namespacesTried,const Na
     if (j==std::string::npos)
     {
       //printf("candidate %s\n",qPrint(aliasInfo.alias));
-      auto candidate = replaceNamespaceAliasesRec(namespacesTried,QCString(aliasInfo.alias));
+      auto candidate = replaceNamespaceAliasesRec(namespacesTried,aliasInfo.alias);
       auto nd = Doxygen::namespaceLinkedMap->find(candidate);
       if (nd)
       {
