@@ -33,6 +33,7 @@
 #include "docbookgen.h"
 #include "xmlgen.h"
 #include "devnullgen.h"
+#include "docoptions.h"
 
 class OutputCodeList;
 
@@ -370,20 +371,17 @@ class OutputList
 
     void generateDoc(const QCString &fileName,int startLine,
                      const Definition *ctx,const MemberDef *md,const QCString &docStr,
-                     bool indexWords,bool isExample,const QCString &exampleName,
-                     bool singleLine,bool linkFromIndex,
-                     bool markdownSupport=Config_getBool(MARKDOWN_SUPPORT),
-                     bool autolinkSupport=Config_getBool(AUTOLINK_SUPPORT));
+                     const DocOptions &options);
 
-    void startFile(const QCString &name,const QCString &manName,const QCString &title, int hierarchyLevel=0);
+    void startFile(const QCString &name, bool isSource,const QCString &manName,const QCString &title, int hierarchyLevel=0);
     void parseText(const QCString &textStr);
 
     //////////////////////////////////////////////////
     // OutputGenIntf implementation
     //////////////////////////////////////////////////
 
-    void writeDoc(const IDocNodeAST *ast,const Definition *ctx,const MemberDef *md)
-    { foreach(&OutputGenIntf::writeDoc,ast,ctx,md,m_id); }
+    void writeDoc(const IDocNodeAST *ast,const Definition *ctx,const MemberDef *md,int sectionLevel=-1)
+    { foreach(&OutputGenIntf::writeDoc,ast,ctx,md,m_id,sectionLevel); }
     void startIndexSection(IndexSection is)
     { foreach(&OutputGenIntf::startIndexSection,is); }
     void endIndexSection(IndexSection is)
@@ -757,6 +755,10 @@ class OutputList
     { foreach(&OutputGenIntf::startPlainFile,name); }
     void endPlainFile()
     { foreach(&OutputGenIntf::endPlainFile); }
+    void startEmbeddedDoc(int indent)
+    { foreach(&OutputGenIntf::startEmbeddedDoc,indent); }
+    void endEmbeddedDoc()
+    { foreach(&OutputGenIntf::endEmbeddedDoc); }
 
   private:
     void newId();
