@@ -17,34 +17,27 @@
 #define PAGEDEF_H
 
 #include "definition.h"
-#include "sortdict.h"
 
-class PageSDict;
+class PageLinkedRefMap;
 class OutputList;
-class FTextStream;
 
 /** @brief A model of a page symbol. */
 class PageDef : public DefinitionMutable, public Definition
 {
   public:
-    virtual ~PageDef() {}
+    ABSTRACT_BASE_CLASS(PageDef)
 
     // setters
-    virtual void setFileName(const char *name) = 0;
+    virtual void setFileName(const QCString &name) = 0;
     virtual void setLocalToc(const LocalToc &tl) = 0;
     virtual void setShowLineNo(bool) = 0;
+    virtual void setTitle(const QCString &title) = 0;
 
     // getters
-    virtual DefType definitionType() const = 0;
-    virtual bool isLinkableInProject() const = 0;
-    virtual bool isLinkable() const = 0;
-    virtual QCString getOutputFileBase() const = 0;
-    virtual QCString anchor() const = 0;
     virtual void findSectionsInDocumentation() = 0;
     virtual QCString title() const = 0;
-    virtual GroupDef *  getGroupDef() const = 0;
-    virtual PageSDict * getSubPages() const = 0;
-    virtual void addInnerCompound(const Definition *) = 0;
+    virtual const GroupDef *getGroupDef() const = 0;
+    virtual const PageLinkedRefMap &getSubPages() const = 0;
     virtual bool visibleInIndex() const = 0;
     virtual bool documentedPage() const = 0;
     virtual bool hasSubPages() const = 0;
@@ -53,17 +46,19 @@ class PageDef : public DefinitionMutable, public Definition
     virtual LocalToc localToc() const = 0;
     virtual void setPageScope(Definition *) = 0;
     virtual Definition *getPageScope() const = 0;
-    virtual QCString displayName(bool=TRUE) const = 0;
     virtual bool showLineNo() const = 0;
 
     virtual void writeDocumentation(OutputList &) = 0;
-    virtual void writeTagFile(FTextStream &) = 0;
+    virtual void writeTagFile(TextStream &) = 0;
     virtual void setNestingLevel(int) = 0;
-    virtual void writePageDocumentation(OutputList &) = 0;
+    virtual void writePageDocumentation(OutputList &) const = 0;
+    virtual void addSectionsToIndex() = 0;
+    virtual void addListReferences() = 0;
+    virtual void addRequirementReferences() = 0;
 
 };
 
-PageDef *createPageDef(const char *f,int l,const char *n,const char *d,const char *t);
+std::unique_ptr<PageDef> createPageDef(const QCString &f,int l,const QCString &n,const QCString &d,const QCString &t);
 
 // --- Cast functions
 
@@ -72,16 +67,12 @@ const PageDef      *toPageDef(const Definition *d);
 
 // ------------------
 
-class PageSDict : public SDict<PageDef>
+class PageLinkedMap : public LinkedMap<PageDef>
+{
+};
+
+class PageLinkedRefMap : public LinkedRefMap<PageDef>
 {
-  public:
-    PageSDict(uint size) : SDict<PageDef>(size) {}
-    virtual ~PageSDict() {}
-  private:
-    int compareValues(const PageDef *i1,const PageDef *i2) const
-    {
-      return qstricmp(i1->name(),i2->name());
-    }
 };
 
 #endif
