@@ -1,12 +1,10 @@
 /******************************************************************************
  *
- * $Id: pycode.h,v 1.9 2001/03/19 19:27:39 root Exp $
- *
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ * Copyright (C) 1997-2020 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -25,18 +23,32 @@
 #ifndef PYCODE_H
 #define PYCODE_H
 
-#include "qtbc.h"
-#include <stdio.h>
+#include "parserintf.h"
 
-class CodeOutputInterface;
 class FileDef;
 class MemberDef;
+class QCString;
 class Definition;
 
-extern void parsePythonCode(CodeOutputInterface &,const char *,const QCString &, 
-             bool ,const char *,FileDef *fd,
-	     int startLine,int endLine,bool inlineFragment,
-             MemberDef *memberDef,bool showLineNumbers,Definition *searchCtx);
-extern void resetPythonCodeParserState();
+class PythonCodeParser : public CodeParserInterface
+{
+  public:
+    PythonCodeParser();
+   ~PythonCodeParser() override;
+    NON_COPYABLE(PythonCodeParser)
+
+    void parseCode(OutputCodeList &codeOutIntf,
+                   const QCString &scopeName,
+                   const QCString &input,
+                   SrcLangExt lang,
+                   bool stripCodeComments,
+                   const CodeParserOptions &options
+                  ) override;
+    void resetCodeParserState() override;
+  private:
+    struct Private;
+    std::unique_ptr<Private> p;
+};
+
 
 #endif
