@@ -24,6 +24,7 @@
 #include "msc.h"
 #include "util.h"
 #include "rtfstyle.h"
+#include "rtfgen.h"
 #include "message.h"
 #include "parserintf.h"
 #include "msc.h"
@@ -35,6 +36,9 @@
 #include "plantuml.h"
 #include "fileinfo.h"
 #include "portable.h"
+#include "codefragment.h"
+#include "cite.h"
+#include "md5.h"
 
 //#define DBG_RTF(x) m_t << x
 #define DBG_RTF(x) do {} while(0)
@@ -53,9 +57,9 @@ static QCString align(const DocHtmlCell &cell)
   return "";
 }
 
-RTFDocVisitor::RTFDocVisitor(TextStream &t,CodeOutputInterface &ci,
-                             const QCString &langExt)
-  : m_t(t), m_ci(ci), m_langExt(langExt)
+RTFDocVisitor::RTFDocVisitor(TextStream &t,OutputCodeList &ci,
+                             const QCString &langExt, int hierarchyLevel)
+  : m_t(t), m_ci(ci), m_langExt(langExt), m_hierarchyLevel(hierarchyLevel)
 {
 }
 
@@ -66,6 +70,19 @@ QCString RTFDocVisitor::getStyle(const QCString &name)
   return sd.reference();
 }
 
+QCString RTFDocVisitor::getListTable(const int id)
+{
+  for (int i=0 ; rtf_Table_Default[i].definition ; i++ )
+  {
+    if ((id == rtf_Table_Default[i].id) && (m_indentLevel == rtf_Table_Default[i].lvl))
+    {
+      return rtf_Table_Default[i].place;
+    }
+  }
+  ASSERT(0);
+  return "";
+}
+
 int RTFDocVisitor::indentLevel() const
 {
   return std::min(m_indentLevel,maxIndentLevels-1);
@@ -76,7 +93,7 @@ void RTFDocVisitor::incIndentLevel()
   m_indentLevel++;
   if (m_indentLevel>=maxIndentLevels)
   {
-    err("Maximum indent level (%d) exceeded while generating RTF output!\n",maxIndentLevels-1);
+    err("Maximum indent level ({}) exceeded while generating RTF output!\n",maxIndentLevels-1);
   }
 }
 
@@ -133,7 +150,7 @@ void RTFDocVisitor::operator()(const DocSymbol &s)
   }
   else
   {
-    err("RTF: non supported HTML-entity found: %s\n",HtmlEntityMapper::instance().html(s.symbol(),TRUE));
+    err("RTF: non supported HTML-entity found: {}\n",HtmlEntityMapper::instance().html(s.symbol(),TRUE));
   }
   m_lastIsPara=FALSE;
 }
@@ -248,6 +265,8 @@ void RTFDocVisitor::operator()(const DocStyleChange &s)
     case DocStyleChange::Italic:
       if (s.enable()) m_t << "{\\i ";     else m_t << "} ";
       break;
+    case DocStyleChange::Kbd:
+    case DocStyleChange::Typewriter:
     case DocStyleChange::Code:
       if (s.enable()) m_t << "{\\f2 ";   else m_t << "} ";
       break;
@@ -304,7 +323,8 @@ void RTFDocVisitor::operator()(const DocVerbatim &s)
       m_t << "\\par\n";
       m_t << rtf_Style_Reset << getStyle("CodeExample");
       getCodeParser(lang).parseCode(m_ci,s.context(),s.text(),langExt,
-                                    s.isExample(),s.exampleFile());
+                                    Config_getBool(STRIP_CODE_COMMENTS),
+                                    CodeParserOptions().setExample(s.isExample(),s.exampleFile()));
       //m_t << "\\par\n";
       m_t << "}\n";
       break;
@@ -338,71 +358,47 @@ void RTFDocVisitor::operator()(const DocVerbatim &s)
       break;
     case DocVerbatim::Dot:
       {
-        static int dotindex = 1;
-        QCString fileName(4096);
-
-        fileName.sprintf("%s%d%s",
-            qPrint(Config_getString(RTF_OUTPUT)+"/inline_dotgraph_"),
-            dotindex++,
-            ".dot"
-           );
-        std::ofstream file = Portable::openOutputStream(fileName);
-        if (!file.is_open())
+        bool exists = false;
+        auto fileName = writeFileContents(Config_getString(RTF_OUTPUT)+"/inline_dotgraph_", // baseName
+                                          ".dot",                                           // extension
+                                          s.text(),                                         // contents
+                                          exists);
+        if (!fileName.isEmpty())
         {
-          err("Could not open file %s for writing\n",qPrint(fileName));
+          writeDotFile(fileName, s.hasCaption(), s.srcFile(), s.srcLine(), !exists);
+          visitChildren(s);
+          includePicturePostRTF(true, s.hasCaption());
         }
-        else
-        {
-          QCString stext = s.text();
-          file.write( stext.data(), stext.length() );
-          file.close();
-        }
-
-        writeDotFile(fileName, s.hasCaption(), s.srcFile(), s.srcLine());
-        visitChildren(s);
-        includePicturePostRTF(true, s.hasCaption());
-
-        if (Config_getBool(DOT_CLEANUP)) Dir().remove(fileName.str());
       }
       break;
     case DocVerbatim::Msc:
       {
-        static int mscindex = 1;
-        QCString baseName(4096);
-
-        baseName.sprintf("%s%d%s",
-            qPrint(Config_getString(RTF_OUTPUT)+"/inline_mscgraph_"),
-            mscindex++,
-            ".msc"
-           );
-        std::ofstream file = Portable::openOutputStream(baseName);
-        if (!file.is_open())
+        bool exists = false;
+        auto fileName = writeFileContents(Config_getString(RTF_OUTPUT)+"/inline_mscgraph_", // baseName
+                                          ".msc",                                           // extension
+                                          "msc {"+s.text()+"}",                             // contents
+                                          exists);
+        if (!fileName.isEmpty())
         {
-          err("Could not open file %s for writing\n",qPrint(baseName));
+          writeMscFile(fileName, s.hasCaption(), s.srcFile(), s.srcLine(), !exists);
+          visitChildren(s);
+          includePicturePostRTF(true, s.hasCaption());
         }
-        QCString text = "msc {";
-        text+=s.text();
-        text+="}";
-        file.write( text.data(), text.length() );
-        file.close();
-
-        writeMscFile(baseName, s.hasCaption(), s.srcFile(), s.srcLine());
-        visitChildren(s);
-        includePicturePostRTF(true, s.hasCaption());
-
-        if (Config_getBool(DOT_CLEANUP)) Dir().remove(baseName.str());
       }
       break;
     case DocVerbatim::PlantUML:
       {
         QCString rtfOutput = Config_getString(RTF_OUTPUT);
-        QCString baseName = PlantumlManager::instance().writePlantUMLSource(
+        auto baseNameVector = PlantumlManager::instance().writePlantUMLSource(
                        rtfOutput,s.exampleFile(),s.text(),PlantumlManager::PUML_BITMAP,
-                       s.engine(),s.srcFile(),s.srcLine());
+                       s.engine(),s.srcFile(),s.srcLine(),true);
 
-        writePlantUMLFile(baseName, s.hasCaption());
-        visitChildren(s);
-        includePicturePostRTF(true, s.hasCaption());
+        for (const auto &baseName: baseNameVector)
+        {
+          writePlantUMLFile(baseName, s.hasCaption());
+          visitChildren(s);
+          includePicturePostRTF(true, s.hasCaption());
+        }
       }
       break;
   }
@@ -444,20 +440,16 @@ void RTFDocVisitor::operator()(const DocInclude &inc)
          m_t << "\\par\n";
          m_t << rtf_Style_Reset << getStyle("CodeExample");
          FileInfo cfi( inc.file().str() );
-         FileDef *fd = createFileDef( cfi.dirPath(), cfi.fileName() );
+         auto fd = createFileDef( cfi.dirPath(), cfi.fileName() );
          getCodeParser(inc.extension()).parseCode(m_ci,inc.context(),
-                                           inc.text(),
-                                           langExt,
-                                           inc.isExample(),
-                                           inc.exampleFile(),
-                                           fd,   // fileDef,
-                                           -1,    // start line
-                                           -1,    // end line
-                                           FALSE, // inline fragment
-                                           0,     // memberDef
-                                           TRUE   // show line numbers
-					   );
-         delete fd;
+                                                  inc.text(),
+                                                  langExt,
+                                                  inc.stripCodeComments(),
+                                                  CodeParserOptions()
+                                                  .setExample(inc.isExample(),inc.exampleFile())
+                                                  .setFileDef(fd.get())
+                                                  .setInlineFragment(true)
+                                                 );
          m_t << "\\par";
          m_t << "}\n";
       }
@@ -467,15 +459,13 @@ void RTFDocVisitor::operator()(const DocInclude &inc)
       m_t << "\\par\n";
       m_t << rtf_Style_Reset << getStyle("CodeExample");
       getCodeParser(inc.extension()).parseCode(m_ci,inc.context(),
-                                        inc.text(),langExt,inc.isExample(),
-                                        inc.exampleFile(),
-                                        0,     // fileDef
-                                        -1,    // startLine
-                                        -1,    // endLine
-                                        TRUE,  // inlineFragment
-                                        0,     // memberDef
-                                        FALSE  // show line numbers
-				       );
+                                               inc.text(),langExt,
+                                               inc.stripCodeComments(),
+                                               CodeParserOptions()
+                                               .setExample(inc.isExample(),inc.exampleFile())
+                                               .setInlineFragment(true)
+                                               .setShowLineNumbers(false)
+                                              );
       m_t << "\\par";
       m_t << "}\n";
       break;
@@ -499,47 +489,20 @@ void RTFDocVisitor::operator()(const DocInclude &inc)
       m_t << "}\n";
       break;
     case DocInclude::Snippet:
+    case DocInclude::SnippetWithLines:
       m_t << "{\n";
       if (!m_lastIsPara) m_t << "\\par\n";
       m_t << rtf_Style_Reset << getStyle("CodeExample");
-      getCodeParser(inc.extension()).parseCode(m_ci,
-                                        inc.context(),
-                                        extractBlock(inc.text(),inc.blockId()),
-                                        langExt,
-                                        inc.isExample(),
-                                        inc.exampleFile()
-                                       );
+      CodeFragmentManager::instance().parseCodeFragment(m_ci,
+                                                        inc.file(),
+                                                        inc.blockId(),
+                                                        inc.context(),
+                                                        inc.type()==DocInclude::SnippetWithLines,
+                                                        inc.trimLeft(),
+                                                        inc.stripCodeComments()
+                                                       );
       m_t << "}";
       break;
-    case DocInclude::SnipWithLines:
-      {
-         FileInfo cfi( inc.file().str() );
-         FileDef *fd = createFileDef( cfi.dirPath(), cfi.fileName() );
-         m_t << "{\n";
-         if (!m_lastIsPara) m_t << "\\par\n";
-         m_t << rtf_Style_Reset << getStyle("CodeExample");
-         getCodeParser(inc.extension()).parseCode(m_ci,
-                                           inc.context(),
-                                           extractBlock(inc.text(),inc.blockId()),
-                                           langExt,
-                                           inc.isExample(),
-                                           inc.exampleFile(),
-                                           fd,
-                                           lineBlock(inc.text(),inc.blockId()),
-                                           -1,    // endLine
-                                           FALSE, // inlineFragment
-                                           0,     // memberDef
-                                           TRUE   // show line number
-                                          );
-         delete fd;
-         m_t << "}";
-      }
-      break;
-    case DocInclude::SnippetDoc:
-    case DocInclude::IncludeDoc:
-      err("Internal inconsistency: found switch SnippetDoc / IncludeDoc in file: %s"
-          "Please create a bug report\n",__FILE__);
-      break;
   }
   m_lastIsPara=TRUE;
 }
@@ -568,7 +531,7 @@ void RTFDocVisitor::operator()(const DocIncOperator &op)
     m_hide = popHidden();
     if (!m_hide)
     {
-      FileDef *fd = 0;
+      std::unique_ptr<FileDef> fd = nullptr;
       if (!op.includeFileName().isEmpty())
       {
         FileInfo cfi( op.includeFileName().str() );
@@ -576,15 +539,13 @@ void RTFDocVisitor::operator()(const DocIncOperator &op)
       }
 
       getCodeParser(locLangExt).parseCode(m_ci,op.context(),op.text(),langExt,
-                                        op.isExample(),op.exampleFile(),
-                                        fd,     // fileDef
-                                        op.line(),    // startLine
-                                        -1,    // endLine
-                                        FALSE, // inline fragment
-                                        0,     // memberDef
-                                        op.showLineNo()  // show line numbers
-                                       );
-      if (fd) delete fd;
+                                          op.stripCodeComments(),
+                                          CodeParserOptions()
+                                          .setExample(op.isExample(),op.exampleFile())
+                                          .setFileDef(fd.get())
+                                          .setStartLine(op.line())
+                                          .setShowLineNumbers(op.showLineNo())
+                                         );
     }
     pushHidden(m_hide);
     m_hide=TRUE;
@@ -643,21 +604,21 @@ void RTFDocVisitor::operator()(const DocCite &cite)
 {
   if (m_hide) return;
   DBG_RTF("{\\comment RTFDocVisitor::operator()(const DocCite &)}\n");
+  auto opt = cite.option();
   if (!cite.file().isEmpty())
   {
-    startLink(cite.ref(),cite.file(),cite.anchor());
-  }
-  else
-  {
-    m_t << "{\\b ";
-  }
-  filter(cite.text());
-  if (!cite.file().isEmpty())
-  {
-    endLink(cite.ref());
+    if (!opt.noCite()) startLink(cite.ref(),cite.file(),cite.anchor());
+
+    filter(cite.getText());
+
+    if (!opt.noCite()) endLink(cite.ref());
   }
   else
   {
+    m_t << "{\\b";
+    if (!opt.noPar()) filter("[");
+    filter(cite.target());
+    if (!opt.noPar()) filter("]");
     m_t << "}";
   }
 }
@@ -674,6 +635,7 @@ void RTFDocVisitor::operator()(const DocAutoList &l)
   m_t << "{\n";
   int level = indentLevel();
   m_listItemInfo[level].isEnum = l.isEnumList();
+  m_listItemInfo[level].isCheck = l.isCheckedList();
   m_listItemInfo[level].type   = '1';
   m_listItemInfo[level].number = 1;
   m_lastIsPara=FALSE;
@@ -681,16 +643,21 @@ void RTFDocVisitor::operator()(const DocAutoList &l)
   if (!m_lastIsPara) m_t << "\\par";
   m_t << "}\n";
   m_lastIsPara=TRUE;
-  if (indentLevel()==0) m_t << "\\par\n";
+  if (!l.isCheckedList() && indentLevel()==0) m_t << "\\par\n";
 }
 
 void RTFDocVisitor::operator()(const DocAutoListItem &li)
 {
+  static int prevLevel = -1;
   if (m_hide) return;
   DBG_RTF("{\\comment RTFDocVisitor::operator()(const DocAutoListItem &)}\n");
-  if (!m_lastIsPara) m_t << "\\par\n";
-  m_t << rtf_Style_Reset;
   int level = indentLevel();
+  if ((level != prevLevel-1) &&
+      (!(level==prevLevel && level != 0 && m_listItemInfo[level].isCheck)) &&
+      (!m_lastIsPara))
+    m_t << "\\par\n";
+  prevLevel= level;
+  m_t << rtf_Style_Reset;
   if (m_listItemInfo[level].isEnum)
   {
     m_t << getStyle("ListEnum") << "\n";
@@ -699,10 +666,22 @@ void RTFDocVisitor::operator()(const DocAutoListItem &li)
   }
   else
   {
-    m_t << getStyle("ListBullet") << "\n";
+    switch (li.itemNumber())
+    {
+      case DocAutoList::Unchecked: // unchecked
+        m_t << getListTable(2) << "\n";
+        break;
+      case DocAutoList::Checked_x: // checked with x
+      case DocAutoList::Checked_X: // checked with X
+        m_t << getListTable(3) << "\n";
+        break;
+      default:
+        m_t << getListTable(1) << "\n";
+        break;
+    }
   }
   incIndentLevel();
-  m_lastIsPara=FALSE;
+  m_lastIsPara=false;
   visitChildren(li);
   decIndentLevel();
 }
@@ -777,6 +756,8 @@ void RTFDocVisitor::operator()(const DocSimpleSect &s)
       m_t << theTranslator->trRemarks(); break;
     case DocSimpleSect::Attention:
       m_t << theTranslator->trAttention(); break;
+    case DocSimpleSect::Important:
+      m_t << theTranslator->trImportant(); break;
     case DocSimpleSect::User: break;
     case DocSimpleSect::Rcs: break;
     case DocSimpleSect::Unknown:  break;
@@ -824,7 +805,8 @@ void RTFDocVisitor::operator()(const DocSimpleList &l)
   if (m_hide) return;
   DBG_RTF("{\\comment RTFDocVisitor::operator()(const DocSimpleSect &)}\n");
   m_t << "{\n";
-  m_listItemInfo[indentLevel()].isEnum = FALSE;
+  m_listItemInfo[indentLevel()].isEnum = false;
+  m_listItemInfo[indentLevel()].isCheck = false;
   m_lastIsPara=FALSE;
   visitChildren(l);
   if (!m_lastIsPara) m_t << "\\par\n";
@@ -857,15 +839,23 @@ void RTFDocVisitor::operator()(const DocSection &s)
   m_t << "{{" // start section
       << rtf_Style_Reset;
   QCString heading;
-  int level = std::min(s.level()+1,4);
+  int level = std::min(s.level()+2+m_hierarchyLevel,4);
+  if (level <= 0)
+    level = 1;
   heading.sprintf("Heading%d",level);
   // set style
   m_t << rtf_Style[heading.str()].reference() << "\n";
   // make table of contents entry
-  filter(s.title());
+  if (s.title())
+  {
+    std::visit(*this,*s.title());
+  }
   m_t << "\n\\par" << "}\n";
   m_t << "{\\tc\\tcl" << level << " \\v ";
-  filter(s.title());
+  if (s.title())
+  {
+    std::visit(*this,*s.title());
+  }
   m_t << "}\n";
   m_lastIsPara=TRUE;
   visitChildren(s);
@@ -880,6 +870,7 @@ void RTFDocVisitor::operator()(const DocHtmlList &l)
   m_t << "{\n";
   int level = indentLevel();
   m_listItemInfo[level].isEnum = l.type()==DocHtmlList::Ordered;
+  m_listItemInfo[level].isCheck = false;
   m_listItemInfo[level].number = 1;
   m_listItemInfo[level].type   = '1';
   for (const auto &opt : l.attribs())
@@ -890,7 +881,7 @@ void RTFDocVisitor::operator()(const DocHtmlList &l)
     }
     if (opt.name=="start")
     {
-      bool ok;
+      bool ok = false;
       int val = opt.value.toInt(&ok);
       if (ok) m_listItemInfo[level].number = val;
     }
@@ -914,7 +905,7 @@ void RTFDocVisitor::operator()(const DocHtmlListItem &l)
     {
       if (opt.name=="value")
       {
-        bool ok;
+        bool ok = false;
         int val = opt.value.toInt(&ok);
         if (ok) m_listItemInfo[level].number = val;
       }
@@ -1030,7 +1021,7 @@ void RTFDocVisitor::operator()(const DocHtmlRow &r)
 {
   if (m_hide) return;
   DBG_RTF("{\\comment RTFDocVisitor::operator()(const DocHtmlRow &)}\n");
-  size_t i,columnWidth=r.numCells()>0 ? rtf_pageWidth/r.numCells() : 10;
+  size_t columnWidth=r.numCells()>0 ? rtf_pageWidth/r.numCells() : 10;
   m_t << "\\trowd \\trgaph108\\trleft-108"
          "\\trbrdrt\\brdrs\\brdrw10 "
          "\\trbrdrl\\brdrs\\brdrw10 "
@@ -1038,7 +1029,7 @@ void RTFDocVisitor::operator()(const DocHtmlRow &r)
          "\\trbrdrr\\brdrs\\brdrw10 "
          "\\trbrdrh\\brdrs\\brdrw10 "
          "\\trbrdrv\\brdrs\\brdrw10 \n";
-  for (i=0;i<r.numCells();i++)
+  for (size_t i=0;i<r.numCells();i++)
   {
     if (r.isHeading())
     {
@@ -1164,7 +1155,7 @@ void RTFDocVisitor::operator()(const DocHtmlHeader &header)
   m_t << "{" // start section
       << rtf_Style_Reset;
   QCString heading;
-  int level = std::min(header.level(),5);
+  int level = std::clamp(header.level()+m_hierarchyLevel,SectionType::MinLevel,SectionType::MaxLevel);
   heading.sprintf("Heading%d",level);
   // set style
   m_t << rtf_Style[heading.str()].reference();
@@ -1255,27 +1246,78 @@ void RTFDocVisitor::operator()(const DocImage &img)
 void RTFDocVisitor::operator()(const DocDotFile &df)
 {
   DBG_RTF("{\\comment RTFDocVisitor::operator()(const DocDotFile &)}\n");
-  if (!Config_getBool(DOT_CLEANUP)) copyFile(df.file(),Config_getString(RTF_OUTPUT)+"/"+stripPath(df.file()));
-  writeDotFile(df);
-  visitChildren(df);
-  includePicturePostRTF(true, df.hasCaption());
+  bool exists = false;
+  std::string inBuf;
+  if (readInputFile(df.file(),inBuf))
+  {
+    auto fileName = writeFileContents(Config_getString(RTF_OUTPUT)+"/"+stripPath(df.file())+"_", // baseName
+                                      ".dot",                                                    // extension
+                                      inBuf,                                                     // contents
+                                      exists);
+    if (!fileName.isEmpty())
+    {
+      writeDotFile(fileName, df.hasCaption(), df.srcFile(), df.srcLine(), !exists);
+      visitChildren(df);
+      includePicturePostRTF(true, df.hasCaption());
+    }
+  }
 }
 void RTFDocVisitor::operator()(const DocMscFile &df)
 {
   DBG_RTF("{\\comment RTFDocVisitor::operator()(const DocMscFile &)}\n");
-  if (!Config_getBool(DOT_CLEANUP)) copyFile(df.file(),Config_getString(RTF_OUTPUT)+"/"+stripPath(df.file()));
-  writeMscFile(df);
-  visitChildren(df);
-  includePicturePostRTF(true, df.hasCaption());
+  bool exists = false;
+  std::string inBuf;
+  if (readInputFile(df.file(),inBuf))
+  {
+    auto fileName = writeFileContents(Config_getString(RTF_OUTPUT)+"/"+stripPath(df.file())+"_", // baseName
+                                      ".msc",                                                    // extension
+                                      inBuf,                                                     // contents
+                                      exists);
+    if (!fileName.isEmpty())
+    {
+      writeMscFile(fileName, df.hasCaption(), df.srcFile(), df.srcLine(), !exists);
+      visitChildren(df);
+      includePicturePostRTF(true, df.hasCaption());
+    }
+  }
 }
 
 void RTFDocVisitor::operator()(const DocDiaFile &df)
 {
   DBG_RTF("{\\comment RTFDocVisitor::operator()(const DocDiaFile &)}\n");
+  bool exists = false;
+  std::string inBuf;
+  if (readInputFile(df.file(),inBuf))
+  {
+    auto fileName = writeFileContents(Config_getString(RTF_OUTPUT)+"/"+stripPath(df.file())+"_", // baseName
+                                      ".dia",                                                    // extension
+                                      inBuf,                                                     // contents
+                                      exists);
+    if (!fileName.isEmpty())
+    {
+      writeDiaFile(fileName, df.hasCaption(), df.srcFile(), df.srcLine(), !exists);
+      visitChildren(df);
+      includePicturePostRTF(true, df.hasCaption());
+    }
+  }
+}
+
+void RTFDocVisitor::operator()(const DocPlantUmlFile &df)
+{
+  DBG_RTF("{\\comment RTFDocVisitor::operator()(const DocPlantUmlFile &)}\n");
   if (!Config_getBool(DOT_CLEANUP)) copyFile(df.file(),Config_getString(RTF_OUTPUT)+"/"+stripPath(df.file()));
-  writeDiaFile(df);
-  visitChildren(df);
-  includePicturePostRTF(true, df.hasCaption());
+  QCString rtfOutput = Config_getString(RTF_OUTPUT);
+  std::string inBuf;
+  readInputFile(df.file(),inBuf);
+  auto baseNameVector = PlantumlManager::instance().writePlantUMLSource(
+                       rtfOutput,QCString(),inBuf,PlantumlManager::PUML_BITMAP,
+                       QCString(),df.srcFile(),df.srcLine(),false);
+  for(const auto &baseName: baseNameVector)
+  {
+    writePlantUMLFile(baseName, df.hasCaption());
+    visitChildren(df);
+    includePicturePostRTF(true, df.hasCaption());
+  }
 }
 
 void RTFDocVisitor::operator()(const DocLink &lnk)
@@ -1403,7 +1445,6 @@ void RTFDocVisitor::operator()(const DocParamList &pl)
   if (sect && sect->hasTypeSpecifier())  config+=2;
   if (useTable)
   {
-    int i;
     m_t << "\\trowd \\trgaph108\\trleft426\\tblind426"
          "\\trbrdrt\\brdrs\\brdrw10\\brdrcf15 "
          "\\trbrdrl\\brdrs\\brdrw10\\brdrcf15 "
@@ -1411,7 +1452,7 @@ void RTFDocVisitor::operator()(const DocParamList &pl)
          "\\trbrdrr\\brdrs\\brdrw10\\brdrcf15 "
          "\\trbrdrh\\brdrs\\brdrw10\\brdrcf15 "
          "\\trbrdrv\\brdrs\\brdrw10\\brdrcf15 "<< "\n";
-    for (i=0;i<columnPos[config][0];i++)
+    for (int i=0;i<columnPos[config][0];i++)
     {
       m_t << "\\clvertalt\\clbrdrt\\brdrs\\brdrw10\\brdrcf15 "
            "\\clbrdrl\\brdrs\\brdrw10\\brdrcf15 "
@@ -1625,10 +1666,9 @@ void RTFDocVisitor::filter(const QCString &str,bool verbatim)
   if (!str.isEmpty())
   {
     const char *p=str.data();
-    char c;
     while (*p)
     {
-      c=*p++;
+      char c=*p++;
       switch (c)
       {
         case '{':  m_t << "\\{";            break;
@@ -1692,64 +1732,37 @@ void RTFDocVisitor::endLink(const QCString &ref)
   m_lastIsPara=FALSE;
 }
 
-void RTFDocVisitor::writeDotFile(const DocDotFile &df)
-{
-  writeDotFile(df.file(), df.hasCaption(), df.srcFile(), df.srcLine());
-}
 void RTFDocVisitor::writeDotFile(const QCString &filename, bool hasCaption,
-                                 const QCString &srcFile, int srcLine)
+                                 const QCString &srcFile, int srcLine, bool newFile)
 {
-  QCString baseName=filename;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
+  QCString baseName=makeBaseName(filename,".dot");
   QCString outDir = Config_getString(RTF_OUTPUT);
-  writeDotGraphFromFile(filename,outDir,baseName,GOF_BITMAP,srcFile,srcLine);
+  if (newFile) writeDotGraphFromFile(filename,outDir,baseName,GraphOutputFormat::BITMAP,srcFile,srcLine);
   QCString imgExt = getDotImageExtension();
   includePicturePreRTF(baseName + "." + imgExt, true, hasCaption);
 }
 
-void RTFDocVisitor::writeMscFile(const DocMscFile &df)
-{
-  writeMscFile(df.file(), df.hasCaption(), df.srcFile(), df.srcLine());
-}
 void RTFDocVisitor::writeMscFile(const QCString &fileName, bool hasCaption,
-                                 const QCString &srcFile, int srcLine)
+                                 const QCString &srcFile, int srcLine, bool newFile)
 {
-  QCString baseName=fileName;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
+  QCString baseName=makeBaseName(fileName,".msc");
   QCString outDir = Config_getString(RTF_OUTPUT);
-  writeMscGraphFromFile(fileName,outDir,baseName,MSC_BITMAP,srcFile,srcLine);
+  if (newFile) writeMscGraphFromFile(fileName,outDir,baseName,MscOutputFormat::BITMAP,srcFile,srcLine);
   includePicturePreRTF(baseName + ".png", true, hasCaption);
 }
 
-void RTFDocVisitor::writeDiaFile(const DocDiaFile &df)
+void RTFDocVisitor::writeDiaFile(const QCString &fileName, bool hasCaption,
+                                 const QCString &srcFile, int srcLine, bool newFile)
 {
-  QCString baseName=df.file();
-  int i;
-  if ((i=baseName.findRev('/'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
+  QCString baseName=makeBaseName(fileName,".dia");
   QCString outDir = Config_getString(RTF_OUTPUT);
-  writeDiaGraphFromFile(df.file(),outDir,baseName,DIA_BITMAP,df.srcFile(),df.srcLine());
-  includePicturePreRTF(baseName + ".png", true, df.hasCaption());
+  if (newFile) writeDiaGraphFromFile(fileName,outDir,baseName,DiaOutputFormat::BITMAP,srcFile,srcLine);
+  includePicturePreRTF(baseName + ".png", true, hasCaption);
 }
 
 void RTFDocVisitor::writePlantUMLFile(const QCString &fileName, bool hasCaption)
 {
-  QCString baseName=fileName;
-  int i;
-  if ((i=baseName.findRev('/'))!=-1)
-  {
-    baseName=baseName.right(baseName.length()-i-1);
-  }
+  QCString baseName=makeBaseName(fileName,".pu");
   QCString outDir = Config_getString(RTF_OUTPUT);
   PlantumlManager::instance().generatePlantUMLOutput(fileName,outDir,PlantumlManager::PUML_BITMAP);
   includePicturePreRTF(baseName + ".png", true, hasCaption);
