@@ -37,16 +37,8 @@ class SQLCodeParser : public CodeParserInterface
                    const QCString &scopeName,
                    const QCString &input,
                    SrcLangExt,
-                   bool isExampleBlock,
-                   const QCString &exampleName=QCString(),
-                   const FileDef *fileDef=nullptr,
-                   int startLine=-1,
-                   int endLine=-1,
-                   bool inlineFragment=FALSE,
-                   const MemberDef *memberDef=nullptr,
-                   bool showLineNumbers=TRUE,
-                   const Definition *searchCtx=nullptr,
-                   bool collectXRefs=TRUE
+                   bool stripCodeComments,
+                   const CodeParserOptions &options
                   ) override;
     void resetCodeParserState() override;
   private:
