@@ -124,6 +124,7 @@ struct SymbolResolver::Private
     {
       m_fileScope = fileScope;
     }
+    const FileDef *fileScope() const { return m_fileScope; }
 
     QCString          resolvedType;
     const MemberDef  *typeDef = nullptr;
@@ -144,6 +145,7 @@ struct SymbolResolver::Private
                            const QCString &args,        // in
                            bool checkCV,                // in
                            bool insideCode,             // in
+                           bool onlyLinkable,           // in
                            const MemberDef **pTypeDef,  // out
                            QCString *pTemplSpec,        // out
                            QCString *pResolvedType);    // out
@@ -393,6 +395,7 @@ const Definition *SymbolResolver::Private::getResolvedSymbolRec(
            const QCString &args,
            bool checkCV,
            bool insideCode,
+           bool onlyLinkable,
            const MemberDef **pTypeDef,
            QCString *pTemplSpec,
            QCString *pResolvedType)
@@ -523,9 +526,26 @@ const Definition *SymbolResolver::Private::getResolvedSymbolRec(
 
     for (Definition *d : range)
     {
-      if (isCodeSymbol(d->definitionType()))
+      if (isCodeSymbol(d->definitionType()) &&
+          (!onlyLinkable ||
+           d->isLinkable() ||
+           d->isLinkableInProject() ||
+           (d->definitionType()==Definition::TypeFile &&
+            (toFileDef(d))->generateSourceFile()
+           ) // undocumented file that has source code we can link to
+          )
+         )
       {
-         getResolvedSymbol(visitedKeys,scope,d,args,checkCV,insideCode,explicitScopePart,strippedTemplateParams,false,
+        if (d->definitionType()==Definition::TypeMember)
+        {
+          const MemberDef *emd = dynamic_cast<const MemberDef *>(d);
+          if (emd && emd->isEnumValue() && emd->getEnumScope() && emd->getEnumScope()->isStrong() && explicitScopePart.isEmpty())
+          {
+            // skip lookup for strong enum values without explicit scope, see issue #11799
+            continue;
+          }
+        }
+        getResolvedSymbol(visitedKeys,scope,d,args,checkCV,insideCode,explicitScopePart,strippedTemplateParams,false,
             minDistance,bestMatch,bestTypedef,bestTemplSpec,bestResolvedType);
       }
       if  (minDistance==0) break; // we can stop reaching if we already reached distance 0
@@ -838,9 +858,9 @@ void SymbolResolver::Private::getResolvedSymbol(
         }
         std::unique_ptr<ArgumentList> argList = stringToArgumentList(md->getLanguage(),actArgs);
         const ArgumentList &mdAl = md->argumentList();
-        match = matchArguments2(md->getOuterScope(),md->getFileDef(),&mdAl,
-              scope, md->getFileDef(),argList.get(),
-              checkCV,md->getLanguage());
+        match = matchArguments2(md->getOuterScope(),md->getFileDef(),md->typeString(),&mdAl,
+                                scope,              md->getFileDef(),md->typeString(),argList.get(),
+                                checkCV,md->getLanguage());
         AUTO_TRACE_ADD("match={}",match);
       }
 
@@ -1194,7 +1214,8 @@ const Definition *SymbolResolver::Private::followPath(VisitedKeys &visitedKeys,
     // try to resolve the part if it is a typedef
     const MemberDef *memTypeDef=nullptr;
     QCString qualScopePart = substTypedef(visitedKeys,current,path.mid(is,l),&memTypeDef);
-    AUTO_TRACE_ADD("qualScopePart={}",qualScopePart);
+    AUTO_TRACE_ADD("qualScopePart={} memTypeDef={}",qualScopePart,memTypeDef?memTypeDef->name():"");
+    const Definition *next = nullptr;
     if (memTypeDef)
     {
       const ClassDef *type = newResolveTypedef(visitedKeys,m_fileScope,memTypeDef,nullptr,nullptr,nullptr);
@@ -1204,7 +1225,14 @@ const Definition *SymbolResolver::Private::followPath(VisitedKeys &visitedKeys,
         return type;
       }
     }
-    const Definition *next = current->findInnerCompound(qualScopePart);
+    else if (m_fileScope)
+    {
+      next = endOfPathIsUsedClass(m_fileScope->getUsedDefinitions(),qualScopePart);
+    }
+    if (next==nullptr)
+    {
+      next = current->findInnerCompound(qualScopePart);
+    }
     AUTO_TRACE_ADD("Looking for {} inside {} result={}",
         qualScopePart, current->name(), next?next->name():QCString());
     if (next==nullptr)
@@ -1612,7 +1640,9 @@ const ClassDef *SymbolResolver::resolveClass(const Definition *scope,
       scope?scope->name():QCString(), name, mayBeUnlinkable, mayBeHidden);
   p->reset();
 
-  auto lang = scope ? scope->getLanguage() : SrcLangExt::Cpp;
+  auto lang = scope ? scope->getLanguage() :
+          p->fileScope() ? p->fileScope()->getLanguage() :
+          SrcLangExt::Cpp;  // fallback to C++
 
   if (scope==nullptr ||
       (scope->definitionType()!=Definition::TypeClass &&
@@ -1658,14 +1688,15 @@ const Definition *SymbolResolver::resolveSymbol(const Definition *scope,
                                                 const QCString &name,
                                                 const QCString &args,
                                                 bool checkCV,
-                                                bool insideCode)
+                                                bool insideCode,
+                                                bool onlyLinkable)
 {
   AUTO_TRACE("scope={} name={} args={} checkCV={} insideCode={}",
              scope?scope->name():QCString(), name, args, checkCV, insideCode);
   p->reset();
   if (scope==nullptr) scope=Doxygen::globalScope;
   VisitedKeys visitedKeys;
-  const Definition *result = p->getResolvedSymbolRec(visitedKeys,scope,name,args,checkCV,insideCode,&p->typeDef,&p->templateSpec,&p->resolvedType);
+  const Definition *result = p->getResolvedSymbolRec(visitedKeys,scope,name,args,checkCV,insideCode,onlyLinkable,&p->typeDef,&p->templateSpec,&p->resolvedType);
   AUTO_TRACE_EXIT("result={}{}", qPrint(result?result->qualifiedName():QCString()),
                                  qPrint(result && result->definitionType()==Definition::TypeMember ? toMemberDef(result)->argsString() : QCString()));
   return result;
