@@ -53,7 +53,6 @@ struct TagInfo;
 class PageDef;
 class SectionInfo;
 class Definition;
-class BufStr;
 class FileInfo;
 class Dir;
 
@@ -162,13 +161,15 @@ void writePageRef(OutputList &ol,const QCString &cn,const QCString &mn);
 
 //QCString getCanonicalTemplateSpec(const Definition *d,const FileDef *fs,const QCString& spec);
 
-bool matchArguments2(const Definition *srcScope,const FileDef *srcFileScope,const ArgumentList *srcAl,
-                     const Definition *dstScope,const FileDef *dstFileScope,const ArgumentList *dstAl,
+bool matchArguments2(const Definition *srcScope,const FileDef *srcFileScope,const QCString &srcReturnType,const ArgumentList *srcAl,
+                     const Definition *dstScope,const FileDef *dstFileScope,const QCString &dstReturnType,const ArgumentList *dstAl,
                      bool checkCV,SrcLangExt lang
                     );
 
 void mergeArguments(ArgumentList &,ArgumentList &,bool forceNameOverwrite=FALSE);
 
+bool matchTemplateArguments(const ArgumentList &srcAl,const ArgumentList &dstAl);
+
 QCString substituteClassNames(const QCString &s);
 
 struct SelectionBlock
@@ -215,6 +216,7 @@ inline bool isIdJS(int c)
 QCString removeRedundantWhiteSpace(const QCString &s);
 
 QCString inlineArgListToDoc(const ArgumentList &al);
+QCString inlineTemplateArgListToDoc(const ArgumentList &al);
 
 QCString argListToString(const ArgumentList &al,bool useCanonicalType=FALSE,bool showDefVals=TRUE);
 
@@ -244,9 +246,9 @@ struct KeywordSubstitution
 
 using KeywordSubstitutionList = std::vector<KeywordSubstitution>;
 
-QCString substituteKeywords(const QCString &s,const KeywordSubstitutionList &keywords);
+QCString substituteKeywords(const QCString &file,const QCString &s,const KeywordSubstitutionList &keywords);
 
-QCString substituteKeywords(const QCString &s,const QCString &title,
+QCString substituteKeywords(const QCString &file,const QCString &s,const QCString &title,
          const QCString &projName,const QCString &projNum,const QCString &projBrief);
 
 int getPrefixIndex(const QCString &name);
@@ -270,11 +272,11 @@ QCString stripScope(const QCString &name);
 QCString convertToId(const QCString &s);
 QCString correctId(const QCString &s);
 
-QCString convertToHtml(const QCString &s,bool keepEntities=TRUE);
+QCString convertToHtml(const QCString &s,bool keepEntities=true);
 
-QCString convertToXML(const QCString &s, bool keepEntities=FALSE);
+QCString convertToXML(const QCString &s, bool keepEntities=false);
 
-QCString convertToJSString(const QCString &s);
+QCString convertToJSString(const QCString &s,bool keepEntities=false,bool singleQuotes=false);
 
 QCString getOverloadDocs();
 
@@ -346,7 +348,7 @@ QCString stripExtensionGeneral(const QCString &fName, const QCString &ext);
 
 QCString stripExtension(const QCString &fName);
 
-//void replaceNamespaceAliases(QCString &scope,size_t i);
+QCString makeBaseName(const QCString &name, const QCString &ext);
 
 int computeQualifiedIndex(const QCString &name);
 
@@ -377,6 +379,7 @@ void addCodeOnlyMappings();
 bool checkIfTypedef(const Definition *scope,const FileDef *fileScope,const QCString &n);
 
 QCString parseCommentAsText(const Definition *scope,const MemberDef *member,const QCString &doc,const QCString &fileName,int lineNr);
+QCString parseCommentAsHtml(const Definition *scope,const MemberDef *member,const QCString &doc,const QCString &fileName,int lineNr);
 
 bool transcodeCharacterStringToUTF8(std::string &input,const char *inputEncoding);
 
@@ -406,6 +409,8 @@ int nextUtf8CharPosition(const QCString &utf8Str,uint32_t len,uint32_t startPos)
 
 void writeMarkerList(OutputList &ol,const std::string &markerText,size_t numMarkers,
                      std::function<void(size_t)> replaceFunc);
+QCString writeMarkerList(const std::string &markerText,size_t numMarkers,
+                     std::function<QCString(size_t)> replaceFunc);
 
 /** Data associated with a HSV colored image. */
 struct ColoredImgDataItem
@@ -432,8 +437,8 @@ QCString processMarkup(const QCString &s);
 
 bool protectionLevelVisible(Protection prot);
 
-QCString stripIndentation(const QCString &s);
-void stripIndentationVerbatim(QCString &doc,const int indentationLevel);
+QCString stripIndentation(const QCString &s,bool skipFirstLine=false);
+void stripIndentationVerbatim(QCString &doc,const int indentationLevel, bool skipFirstLine=true);
 
 QCString getDotImageExtension();
 
@@ -479,4 +484,10 @@ size_t updateColumnCount(const char *s,size_t col);
 QCString mangleCSharpGenericName(const QCString &name);
 QCString demangleCSharpGenericName(const QCString &name,const QCString &templArgs);
 
+QCString extractBeginRawStringDelimiter(const char *rawStart);
+QCString extractEndRawStringDelimiter(const char *rawEnd);
+
+QCString writeFileContents(const QCString &baseName,const QCString &extension,const QCString &content,bool &exists);
+void cleanupInlineGraph();
+
 #endif
